

from nive.tests import __local
from nive.tests import db_app 

from nive.security import User


class TestSecurityContext(object):
    ppp="view"
    def has_permission(self,p,o):
        return p==self.ppp

class containerTest_db:
    
    def setUp(self):
        self._loadApp()
        self.remove=[]

    def tearDown(self):
        self._closeApp(True, True)


    def test_basics(self):
        #print "Testing shortcuts"
        a=self.app
        user = User("test")
        ccc = a.db.GetCountEntries()
        self.assertTrue(a.root)
        self.assertTrue(a.db)
        r = a.root
        o1 = db_app.createObj1(r)
        o2 = db_app.createObj2(o1)
        id1 = o1.id
        id2 = o2.id
        self.remove.append(id1)
        self.assertTrue(a.obj(id1))
        self.assertTrue(r.__getitem__(str(id1)))
        self.assertTrue(r.obj(id1))
        self.assertTrue(o1.obj(id2))
        self.assertTrue(o1.db==a.db)
        self.assertTrue(r.db==a.db)
        self.assertTrue(o1.relpath)
        self.assertTrue(o2.relpath)

        self.assertTrue(r.IsTypeAllowed("type1", user=user))
        self.assertTrue(o1.IsTypeAllowed("type1", user=user))
        
        try:
            o2.IsTypeAllowed("type1", user=user)
            self.assertTrue(False, "Object is not a container: IsTypeAllowed")
        except AttributeError:
            pass
        self.assertTrue(r.GetAllowedTypes(user=user))
        self.assertTrue(o1.GetAllowedTypes(user=user))
        r.Delete(id1, user=user)
        self.assertEqual(ccc, a.db.GetCountEntries())
        

    def test_createroots(self):
        #print "Testing root basics"
        a=self.app
        ccc = a.db.GetCountEntries()
        r=db_app.root(a)
        user = User("test")
        #rootValues()
        self.assertTrue(r.GetID()<=0)
        self.assertTrue(r.GetTypeID())
        self.assertTrue(r.GetTitle())
        self.assertTrue(r.GetPath())
        #rootParents()
        self.assertTrue(len(r.GetParents())==0)
        self.assertTrue(len(r.GetParentTitles())==0)
        self.assertTrue(len(r.GetParentPaths())==0)
        self.assertEqual(ccc, a.db.GetCountEntries())
        

    def test_createobjs_kws(self):
        #print "Testing new object creation, values and delete"
        a=self.app
        r=db_app.root(a)
        # create
        user = User("test")
        type = "type3"
        data = {}
        user = User("test")
        o = r.Create(type, data=data, user=user, custom=123)
        self.assertTrue(o.__kws_test__.get("custom")==123)


    def test_createobjs(self):
        #print "Testing new object creation, values and delete"
        a=self.app
        r=db_app.root(a)
        # create
        user = User("test")
        ccc = a.db.GetCountEntries()

        o1 = db_app.createObj1(r)
        # -> o1
        o3 = db_app.createObj1(o1)
        o5 = db_app.createObj3(o1)
        o4 = db_app.createObj1(o3)
        o6 = db_app.createObj2(o5)

        o2 = db_app.createObj2(r)
        #import pdb
        #pdb.set_trace()

        # todo [3] fix containment non container
        #self.assertRaises(ContainmentError, createObj1, o5)
        
        self.assertTrue(r.IsContainer())
        self.assertTrue(o1.IsContainer())
        self.assertTrue(o2.IsContainer()==False)
        self.assertTrue(o1.GetID())
        self.assertTrue(o1.GetTypeID()=="type1")
        self.assertTrue(o1.GetTitle()=="")
        self.assertTrue(o1.GetPath())
        #Parents()
        self.assertTrue(len(o1.GetParents())==1)
        self.assertTrue(len(o1.GetParentIDs())==1)
        self.assertTrue(len(o1.GetParentTitles())==1)
        self.assertTrue(len(o1.GetParentPaths())==1)
        self.assertTrue(len(o1.GetObjsList())==2)

        self.assertTrue(o4.parent==o3)
        self.assertTrue(len(o4.GetParents())==3)
        self.assertTrue(len(o4.GetParentIDs())==3)
        self.assertTrue(len(o4.GetParentTitles())==3)
        self.assertTrue(len(o4.GetParentPaths())==3)
        self.assertTrue(len(o3.GetObjsList())==1)
        self.assertTrue(o3.GetObj(o4.GetID()))

        self.assertEqual(ccc+6, db_app.statdb(a))

        newO = r.Duplicate(o1, user)
        self.assertTrue(newO)
        self.assertTrue(newO.IsContainer())
        self.assertTrue(newO.GetID())
        self.assertTrue(newO.GetTypeID()=="type1")
        self.assertTrue(newO.GetTitle()=="")
        self.assertTrue(newO.GetPath())
        #Parents()
        self.assertTrue(len(newO.GetParents())==1)
        self.assertTrue(len(newO.GetParentIDs())==1)
        self.assertTrue(len(newO.GetParentTitles())==1)
        self.assertTrue(len(newO.GetParentPaths())==1)
        self.assertTrue(len(newO.GetObjsList())==2, len(newO.GetObjsList()))

        self.assertEqual(ccc+11, db_app.statdb(a))

        o1.Close()
        o2.Close()
        r.Delete(o1.GetID(), user=user)
        r.Delete(o2.GetID(), user=user)
        self.assertEqual(ccc+5, db_app.statdb(a))

        newO.Close()
        r.Delete(newO.GetID(), user=user)
        self.assertEqual(ccc, db_app.statdb(a))
        
    def test_createobjs_simple(self):
        #print "Testing new object creation, values and delete"
        a=self.app
        r=db_app.root(a)
        # create
        user = User("test")
        ccc = a.db.GetCountEntries()

        typedef = a.configurationQuery.GetObjectConf("type1")
        data = db_app.data1_1

        o1 = r.CreateWithoutEventsAndSecurity(typedef, data, user)
        self.assertTrue(o1)
        self.remove.append(o1.id)
        o1.Commit(user)
        self.assertTrue(r.obj(o1.id))

        o2 = r.CreateWithoutEventsAndSecurity(typedef, data, user)
        self.assertTrue(o2)
        self.remove.append(o2.id)
        o2.Commit(user)
        self.assertTrue(r.obj(o2.id))

        o3 = o2.CreateWithoutEventsAndSecurity(typedef, data, user)
        self.assertTrue(o3)
        self.assertTrue(o3.parent.id==o2.id)
        o3.Commit(user)
        self.assertTrue(o2.obj(o3.id))


    def test_lists(self):
        #print "Testing objects and subobjects"
        a=self.app
        r=db_app.root(a)
        ccc = a.db.GetCountEntries()
        user = User("test")
        # errors
        id = 9865898568444
        try:
            r.obj(id)
            self.assertTrue(False)
        except:
            pass
        try:
            r.GetObj(id)
            self.assertTrue(False)
        except:
            pass
        # objects and parents
        self.assertTrue(r.defaultSort)
        ccontainer = len(r.GetObjs(containerOnly=1, batch=False))
        cobjs = len(r.GetObjs(batch=False))
        ccontainer2 = len(r.GetObjsList(containerOnly=1, parameter={"pool_type":"type2"}))
        cobjs2 = len(r.GetObjsList(parameter={"pool_type":"type2"}))
        c=db_app.statdb(a)
        o1 = db_app.createObj1(r)
        self.assertTrue(o1)
        self.remove.append(o1.id)
        o2 = db_app.createObj2(r)
        self.assertTrue(o2)
        self.remove.append(o2.id)
        o3 = db_app.createObj1(o1)
        self.assertTrue(o3)
        o4 = db_app.createObj2(o3)
        self.assertTrue(o4)
        o5 = db_app.createObj2(o3)
        self.assertTrue(o5)
        self.assertEqual(c+5, db_app.statdb(a))
        try:
            db_app.createObj1(o5)
            self.assertTrue(False)
        except:
            pass
        try:
            db_app.createObj2(o5)
            self.assertTrue(False)
        except:
            pass

        # objects
        id = o1.GetID()
        self.assertTrue(r.obj(id))
        self.assertTrue(r.GetObj(id))
        self.assertTrue(o1.GetObj(o3.GetID()))
        self.assertTrue(o3.GetObj(o4.GetID()))
        id = o5.GetID()
        self.assertTrue(r.LookupObj(id))

        # subitems
        #root
        self.assertTrue(len(r.GetObjs(containerOnly=1, batch=False))==ccontainer+2)
        self.assertTrue(len(r.GetObjs(batch=False))==cobjs+2)
        self.assertTrue(len(r.GetObjs(containerOnly=1))==ccontainer+2)  # less failsafe than batch=False. On failure reset testdata.
        self.assertTrue(len(r.GetObjs())==cobjs+2)
        self.assertTrue(len(r.GetObjs(pool_type="type2")))
        self.assertTrue(len(r.GetObjsList(containerOnly=1, parameter={"pool_type":"type2"}))==ccontainer2+1)
        self.assertTrue(len(r.GetObjsList(parameter={"pool_type":"type2"}))==cobjs2+1)
        self.assertTrue(len(o3.GetObjsBatch([o4.id,o5.id])))
        # object
        self.assertTrue(len(o3.GetObjs(containerOnly=1))==2)
        self.assertTrue(len(o3.GetObjs())==2)
        self.assertTrue(len(o3.GetObjs(pool_type="type2"))==2)
        self.assertTrue(len(o3.GetObjsList(containerOnly=1, parameter={"pool_type":"type2"}))==2)
        self.assertTrue(len(o3.GetObjsList(parameter={"pool_type":"type2"}))==2)
        self.assertTrue(len(o3.GetObjsList(containerOnly=1, parameter={"pool_type":"type2"}, operators={"pool_type":"<>"}))==0)
        self.assertTrue(len(o3.GetObjsList(parameter={"pool_type":"type2"}, operators={"pool_type":"<>"}))==0)
        self.assertTrue(len(o1.GetSubtreeIDs())==3)
        
        r.DeleteInternal(o1.GetID(), user=user)
        r.DeleteInternal(o2.GetID(), user=user, obj=o2)
        self.assertEqual(ccc, a.db.GetCountEntries())
        


    def test_restraintsConf(self):
        #print "Testing new object creation, values and delete"
        a=self.app
        r=db_app.root(a)
        
        p,o=r.ObjQueryRestraints(self)
        p1,o1=r.queryRestraints
        self.assertTrue(p==p1)
        self.assertTrue(o==o1)
        
        r.queryRestraints = {"id":123}, {"id":"="}
        p,o=r.ObjQueryRestraints(self)
        self.assertTrue(p=={"id":123})
        self.assertTrue(o=={"id":"="})
        
        r.queryRestraints = {"id":123}, {"id":"="}
        p,o=r.ObjQueryRestraints(self, {"aa":123}, {"aa":"="})
        self.assertTrue(p=={"id":123,"aa":123})
        self.assertTrue(o=={"id":"=","aa":"="})
        
        def callback(**kw):
            kw["parameter"]["id"] = 456
            kw["operators"]["id"] = "LIKE"
        r.ListenEvent("loadRestraints", callback)
        p,o=r.ObjQueryRestraints(self)
        self.assertTrue(p=={"id":456})
        self.assertTrue(o=={"id":"LIKE"})


    def test_restraintsLookup(self):
        #print "Testing objects and subobjects"
        a=self.app
        r=db_app.root(a)
        ccc = a.db.GetCountEntries()
        user = User("test")
        # errors
        id = 9865898568444
        try:
            r.obj(id)
            self.assertTrue(False)
        except:
            pass
        try:
            r.GetObj(id)
            self.assertTrue(False)
        except:
            pass
        # objects and parents
        self.assertTrue(r.defaultSort)
        ccontainer = len(r.GetObjs(containerOnly=1, batch=False))
        cobjs = len(r.GetObjs(batch=False))
        ccontainer2 = len(r.GetObjsList(containerOnly=1, parameter={"pool_type":"type2"}))
        cobjs2 = len(r.GetObjsList(parameter={"pool_type":"type2"}))
        c=db_app.statdb(a)
        o1 = db_app.createObj1(r)
        self.assertTrue(o1)
        self.remove.append(o1.id)
        o2 = db_app.createObj2(r)
        self.assertTrue(o2)
        self.remove.append(o2.id)
        o3 = db_app.createObj1(o1)
        self.assertTrue(o3)
        o4 = db_app.createObj2(o3)
        self.assertTrue(o4)
        o5 = db_app.createObj2(o3)
        self.assertTrue(o5)
        self.assertTrue(c+5==db_app.statdb(a))
        try:
            db_app.createObj1(o5)
            self.assertTrue(False)
        except:
            pass
        try:
            db_app.createObj2(o5)
            self.assertTrue(False)
        except:
            pass

        r.queryRestraints = {"pool_state":99}, {"pool_state":">"}

        # objects
        id = o1.GetID()
        self.assertFalse(r.obj(id))
        self.assertFalse(r.GetObj(id))
        self.assertFalse(o1.GetObj(o3.GetID()))
        self.assertFalse(o3.GetObj(o4.GetID()))
        id = o5.GetID()
        self.assertFalse(r.obj(id))

        # subitems
        #root
        self.assertTrue(len(r.GetObjs(containerOnly=1, batch=False))==0)
        self.assertTrue(len(r.GetObjs(batch=False))==0)
        self.assertTrue(len(r.GetObjs(containerOnly=1))==0)
        self.assertTrue(len(r.GetObjs())==0)
        self.assertTrue(len(r.GetObjsList(containerOnly=1, parameter={"pool_type":"type2"}))==0)
        p,o=r.ObjQueryRestraints(r)
        p.update({"pool_type":"type2"})
        self.assertTrue(len(r.GetObjsList(parameter=p))==0)
        self.assertTrue(len(o3.GetObjsBatch([o4.id,o5.id]))==0)
        # object
        self.assertTrue(len(o3.GetObjs(containerOnly=1))==0)
        self.assertTrue(len(o3.GetObjs())==0)
        self.assertTrue(len(o3.GetObjsList(containerOnly=1, parameter={"pool_type":"type2"}))==0)
        p,o=r.ObjQueryRestraints(self)
        p.update({"pool_type":"type2"})
        self.assertTrue(len(o3.GetObjsList(parameter=p))==0)
        p,o=r.ObjQueryRestraints(self)
        self.assertTrue(len(o3.GetObjsList(containerOnly=1, parameter={"pool_type":"type2"}, operators={"pool_type":"<>"}))==0)
        p,o=r.ObjQueryRestraints(self)
        p.update({"pool_type":"type2"})
        o.update({"pool_type":"<>"})
        self.assertTrue(len(o3.GetObjsList(parameter=p, operators=o))==0)
        
        r.queryRestraints = {}, {}

        r.DeleteInternal(o1.GetID(), user=user)
        r.DeleteInternal(o2.GetID(), user=user, obj=o2)
        self.assertEqual(ccc, a.db.GetCountEntries())
        


    def test_shortcuts(self):
        #print "Testing shortcuts"
        a=self.app
        user = User("test")
        ccc = a.db.GetCountEntries()
        self.assertTrue(a.root)
        self.assertTrue(a.db)
        r = a.root
        #root
        r.app
        r.db
        r.root
        r.GetID()
        r.GetTypeID()
        r.GetTypeName()
        r.GetTitle()
        r.GetPath()
        r.IsRoot()
        r.GetParents()
        r.GetParentIDs()
        r.GetParentTitles()
        r.GetParentPaths()
        r.GetTool("nive.tools.example")
        r.Close()


    def test_shortcuts2(self):
        #print "Testing shortcuts"
        a=self.app
        user = User("test")
        ccc = a.db.GetCountEntries()
        self.assertTrue(a.root)
        self.assertTrue(a.db)
        r = a.root
        o1 = db_app.createObj1(r)
        self.assertTrue(o1)
        self.remove.append(o1.id)
        #root
        o1.app
        o1.db
        o1.root
        o1.GetID()
        o1.GetTypeID()
        o1.GetTypeName()
        o1.GetTitle()
        o1.GetPath()
        o1.IsRoot()
        o1.GetParents()
        o1.GetParentIDs()
        o1.GetParentTitles()
        o1.GetParentPaths()
        o1.GetTool("nive.tools.example")
        o1.Close()
        self.assertEqual(ccc+1, a.db.GetCountEntries())
        r.Delete(o1.id, user)






class containerTest_db_sqlite(containerTest_db, __local.SqliteTestCase):
    """
    see tests.__local
    """

class containerTest_db_mysql(containerTest_db, __local.MySqlTestCase):
    """
    see tests.__local
    """
    
class containerTest_db_pg(containerTest_db, __local.PostgreSqlTestCase):
    """
    see tests.__local
    """
    


from pyramid import testing

class groupsrootTest_db:
    
    def setUp(self):
        self.request = testing.DummyRequest()
        self.request._LOCALE_ = "en"
        self.config = testing.setUp(request=self.request)
        self.request.content_type = ""
        self._loadApp(["nive.extensions.localgroups"])
        self.remove=[]

    def tearDown(self):
        db_app.emptypool(self.app)
        self.app.Close()

    def test_permissions(self):
        #print "Testing shortcuts"
        a=self.app
        user = User("test")
        ccc = a.db.GetCountEntries()
        self.assertTrue(a.root)
        self.assertTrue(a.db)
        r = a.root
        o1 = db_app.createObj1(r)
        self.assertTrue(o1)
        self.remove.append(o1.id)
        #root
        testsec = TestSecurityContext()
        self.assertTrue(a.root.GetObj(self.remove[-1], permission="view", securityContext=testsec))
        # todo: add authentication policy to tests
        #self.assertRaises(PermissionError, a.root.GetObj, self.remove[-1], permission="none", securityContext=testsec)
        #self.assert_(a.root.GetObj(self.remove[-1], permission="none", securityContext=testsec))
        r.Delete(o1.id, user)

    def test_rootsGroups(self):
        a=self.app
        r=db_app.root(a)

        userid = "test"
        r.RemoveLocalGroups(None, None)
        self.assertFalse(r.GetLocalGroups(userid))
        r.AddLocalGroup(userid, "group:local")
        self.assertEqual(r.GetLocalGroups(userid), ["group:local"])
        r.RemoveLocalGroups("nouser", "nogroup")
        self.assertEqual(r.GetLocalGroups(userid), ["group:local"])
        r.RemoveLocalGroups(userid, "nogroup")
        self.assertEqual(r.GetLocalGroups(userid), ["group:local"])
        r.RemoveLocalGroups("nouser", "group:local")
        self.assertEqual(r.GetLocalGroups(userid), ["group:local"])
        r.RemoveLocalGroups(userid, "group:local")
        self.assertFalse(r.GetLocalGroups(userid))
        r.AddLocalGroup(userid, "group:local")
        r.RemoveLocalGroups(userid, None)
        self.assertFalse(r.GetLocalGroups(userid))
        r.db.Undo()
        


class groupsrootTest_db_sqlite(groupsrootTest_db, __local.SqliteTestCase):
    """
    see tests.__local
    """

class groupsrootTest_db_mysql(groupsrootTest_db, __local.MySqlTestCase):
    """
    see tests.__local
    """

class groupsrootTest_db_pg(groupsrootTest_db, __local.PostgreSqlTestCase):
    """
    see tests.__local
    """






# -*- coding: utf-8 -*-

import time
import unittest

from nive.security import User

"""
#totest:
templates/
definitions.py
parts.py
root.py
search.py
view.py
"""


class IfaceTest:#(unittest.TestCase): # TODO tests

	def setUp(self):
		app = App()
		app.SetConfiguration({"objects": [typedef]})
		self.c = IFace(app)


	def testReg(self):
		self.c.RegisterType(typedef)
		self.c.RegisterComponent(viewdef)
		
	def testGet(self):
		container = Ob()
		container.IContainer=1
		object = Ob()
		addtype = "page"
		self.assertTrue(self.c.GetFldsAdd(container, addtype))
		self.assertTrue(self.c.GetFldsEdit(object))
		self.assertTrue(self.c.GetFldsMetaView(object))
		self.assertTrue(self.c.GetFldsDataView(object))

	def testSearch(self):
		container = Ob()
		container.IContainer=1
		self.assertTrue(self.c.GetSearchConf("", container=container))
		self.assertTrue(self.c.GetSearchConf("default", container=container))

	def testF(self):	
		object = Ob()
		self.assertTrue(self.c.GetTabs(object))
		self.assertTrue(self.c.GetShortcuts(object))
		
	def testRedirect(self):	
		object = Ob()
		self.assertTrue(self.c.GetRedirect("delete", view=object))





import unittest
import datetime
import nive.components.reform.widget
from nive.components.reform.schema import deferred

class TestFunctional(unittest.TestCase):
    def _makeSchema(self):
        from nive.components.reform.schema import Schema
        #from nive.components.reform.schema import MappingSchema
        #from nive.components.reform.schema import SequenceSchema
        from nive.components.reform.schema import SchemaNode
        from nive.components.reform.schema import String
        from nive.components.reform.schema import Boolean
        from nive.components.reform.schema import Date
    
        #class DatesSchema(SequenceSchema):
        #    date = SchemaNode(Date())

        #class SeriesSchema(MappingSchema):
        #    name = SchemaNode(String())
        #    dates = DatesSchema()

        class MySchema(Schema):
            name = SchemaNode(String())
            title = SchemaNode(String())
            cool = SchemaNode(Boolean(), default=True, missing=True)
            date = SchemaNode(Date())
            #series = SeriesSchema()

        schema = MySchema()
        return schema

    def _makeForm(self, schema):
        from nive.components.reform.form import Form
        return Form(schema, formid='myform')

    def _soupify(self, html):
        from BeautifulSoup import BeautifulSoup
        return BeautifulSoup(html)

    #<removed def test_render_empty(self):
    #<removed def test_render_not_empty(self):

    def test_widget_deserialize(self):
        filled = {
            'name': 'project1',
            'title': 'Cool project',
            'date': '2008-10-12',
            }
        schema = self._makeSchema()
        form = self._makeForm(schema)
        result = form.widget.deserialize(form, filled)
        expected = {'date': '2008-10-12',
                    'name': 'date series 1',
                    'cool': 'false',
                    'name': 'project1',
                    'title': 'Cool project'}
        self.assertEqual(result, expected)

    def test_validate(self):
        from nive.components.reform.schema import null
        from nive.components.reform.exception import ValidationFailure
        schema = self._makeSchema()
        form = self._makeForm(schema)
        try:
            form.validate([])
        except ValidationFailure as ve:
            e = ve.error

            self.assertEqual(form.error, e)
            self.assertEqual(form.children[0].error, e.children[0])
            self.assertEqual(form.children[1].error, e.children[1])
            #self.assertEqual(form.children[3].error, e.children[2])
            #self.assertEqual(form.children[3].children[0].error,
            #                 e.children[2].children[0])
            self.assertEqual(
                ve.cstruct,
                {
                    'date': null,
                    'cool': 'false',
                    'name': null,
                    'title': null,
                 }
                )
        
@deferred
def deferred_date_validator(node, kw):
    max_date = kw.get('max_date')
    if max_date is None: max_date = datetime.date.today()
    return schema.Range(min=datetime.date.min, max=max_date)

@deferred
def deferred_date_description(node, kw):
    max_date = kw.get('max_date')
    if max_date is None: max_date = datetime.date.today()
    return 'Blog post date (no earlier than %s)' % max_date.ctime()

@deferred
def deferred_date_missing(node, kw):
    default_date = kw.get('default_date')
    if default_date is None: default_date = datetime.date.today()
    return default_date

@deferred
def deferred_body_validator(node, kw):
    max_bodylen = kw.get('max_bodylen')
    if max_bodylen is None:  max_bodylen = 1 << 18
    return schema.Length(max=max_bodylen)

@deferred
def deferred_body_description(node, kw):
    max_bodylen = kw.get('max_bodylen')
    if max_bodylen is None:  max_bodylen = 1 << 18
    return 'Blog post body (no longer than %s bytes)' % max_bodylen

@deferred
def deferred_body_widget(node, kw):
    body_type = kw.get('body_type')
    if body_type == 'richtext':
        widget = reform.widget.RichTextWidget()
    else: # pragma: no cover
        widget = reform.widget.TextAreaWidget()
    return widget

@deferred
def deferred_category_validator(node, kw):
    categories = kw.get('categories', [])
    return schema.OneOf([ x[0] for x in categories ])

@deferred
def deferred_category_widget(node, kw):
    categories = kw.get('categories', [])
    return reform.widget.RadioChoiceWidget(values=categories)

from nive.components.reform import schema
from nive.components import reform

class BlogPostSchema(schema.Schema):
    title = schema.SchemaNode(
        schema.String(),
        title = 'Title',
        description = 'Blog post title',
        validator = schema.Length(min=5, max=100),
        widget = reform.widget.TextInputWidget(),
        )
    date = schema.SchemaNode(
        schema.Date(),
        title = 'Date',
        missing = deferred_date_missing,
        description = deferred_date_description,
        validator = deferred_date_validator,
        widget = reform.widget.DateInputWidget(),
        )
    body = schema.SchemaNode(
        schema.String(),
        title = 'Body',
        description = deferred_body_description,
        validator = deferred_body_validator,
        widget = deferred_body_widget,
        )
    category = schema.SchemaNode(
        schema.String(),
        title = 'Category',
        description = 'Blog post category',
        validator = deferred_category_validator,
        widget = deferred_category_widget,
        )

def remove_date(node, kw):
    if kw.get('nodates'): del node['date']

class TestDeferredFunction(unittest.TestCase):
    def test_it(self):
        schema = BlogPostSchema(after_bind=remove_date).bind(
            max_date = datetime.date.max,
            max_bodylen = 5000,
            body_type = 'richtext',
            default_date = datetime.date.today(),
            categories = [('one', 'One'), ('two', 'Two')]
            )
        self.assertEqual(schema['date'].missing, datetime.date.today())
        self.assertEqual(schema['date'].validator.max, datetime.date.max)
        self.assertEqual(schema['date'].widget.__class__.__name__,
                         'DateInputWidget')
        self.assertEqual(schema['body'].description,
                         'Blog post body (no longer than 5000 bytes)')
        self.assertEqual(schema['body'].validator.max, 5000)
        self.assertEqual(schema['body'].widget.__class__.__name__,
                         'RichTextWidget')
        self.assertEqual(schema['category'].validator.choices, ['one', 'two'])
        self.assertEqual(schema['category'].widget.values,
                         [('one', 'One'), ('two', 'Two')])
        
        
        
        




from nive.tool import Tool, ToolView
from nive.definitions import ToolConf, ViewConf, FieldConf
from nive.definitions import IApplication
from nive.i18n import _

configuration = ToolConf(
    id = "testobjects",
    context = "nive.tools.testObjects.testobjects",
    name = "Test load all objects",
    description = "tests tree structure and obj data",
    apply = (IApplication,),
    mimetype = "text/html",
    data = [
        FieldConf(id="repair", datatype="number", default="", name="Repair parents -> move to id", description=""),
        FieldConf(id="delete", datatype="list", default="0",  name="Delete if non existing data table id",
                  listItems=[dict(id="1",name="Yes"),
                             dict(id="0",name="No")],
                  description=""),
        FieldConf(id="tag", datatype="string", default="testobjects", hidden=1)
    ],
    views = [
        ViewConf(name="", view=ToolView, attr="form", permission="admin", context="nive.tools.testObjects.testobjects")
    ]
)

class testobjects(Tool):
    """
    """

    def _Run(self, **values):

        self.InitStream()

        delete = values.get("delete")=="1"
        repair = values.get("repair")
        if repair:
            repair = int(repair)
        else:
            repair = None

        app = self.app
        root = app.root
        datapool = app.db
        conn = datapool.connection

        if not conn:
            self.stream.write(_(u"Database connection error (${name})\n", mapping={u"name": app.dbConfiguration.context}))
            return self.stream, ""
        
        if not conn.IsConnected():
            self.stream.write(_(u"Database connection error (${name})\n", mapping={u"name": app.dbConfiguration.context}))
            return self.stream, ""

        # load all pool_files
        sql="select id, pool_type, title, pool_unitref, pool_dataref, pool_datatbl from pool_meta"
        cc = conn.cursor()
        cc.execute(sql)
        # loop
        c=e=o=0
        for rec in cc.fetchall():
            c+=1
            try:
                obj = root.LookupObj(rec[0])
                if obj is None:
                    sql = "select id from %s where id=%s"%(rec[5], rec[4])
                    cc.execute(sql)
                    test = cc.fetchall()
                    if not test:
                        self.stream.write("OBJ ERR: #%d (in %d) - %s - %s -> data table entry missing: %s<br>"%(rec[0], rec[3], rec[1], rec[2] or '~', sql))
                        if delete:
                            sql = "delete from pool_meta where id=%s"
                            cc.execute(sql, (rec[0],))
                        else:
                            e += 1
                    elif root.LookupObj(rec[3]) is None:
                        self.stream.write("OBJ ERR: #%d (in %d) - %s - %s -> parent missing: %d<br>" % (rec[0], rec[3], rec[1], rec[2] or '~', rec[3]))
                        if repair is not None:
                            sql = "update pool_meta set pool_unitref=%s where id=%s"
                            cc.execute(sql, (repair, rec[0]))
                        else:
                            e += 1
            except Exception as excp:
                self.stream.write("OBJ EXCP: #%d (in %d) - %s - %s -> %s<br>"%(rec[0], rec[3], rec[1], rec[2] or '~', str(excp)))
                e+=1
                #raise
                continue
            o+=1
        self.stream.write("Done. %d total, %d ok, %d errors <br>"%(c,o,e))

        return self.stream, True




# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
# Released under GPL3. See license.txt
#
import string, time
import logging

from nive.utils.dataPool2.dbManager import DatabaseManager
from nive.definitions import ConfigurationError


try:    
    import psycopg2
except:
    # ignore to make tests imports pass
    # __init__() will raise an import error
    pass


class PostgresManager(DatabaseManager):
    """
    """
    modifyColumns = True

    def __init__(self):
        # might not be imported on startup
        import psycopg2
        self.db = None
        self.dbname = ""
        self.dbConn = None
        self.engine = ""
        self.useUtf8 = True
        self.log = logging.getLogger("dbManager")

    # Database Options ---------------------------------------------------------------------

    def Connect(self, databaseName = "", ip = "", user = "", pw = ""):
        self.dbname = databaseName
        self.dbConn = psycopg2.connect(database = databaseName, host = ip, user = user, passwd = pw)
        self.db = self.dbConn.cursor()
        return self.db is not None

    # Options ----------------------------------------------------------------

    def ModifyColumn(self, tableName, columnName, newColumnName, columnOptions):
        if not self.IsDB():
            return False
        #print("alter table %s change %s %s %s" % (tableName, columnName, newColumnName, columnOptions))
        if not columnOptions or not columnName or not tableName:
            return False
        if newColumnName:
            self.log.debug("alter table %s rename column %s to %s" % (tableName, columnName, newColumnName))
            self.db.execute("alter table %s rename column %s to %s" % (tableName, columnName, newColumnName))
        opt = columnOptions.split(" ")
        self.log.debug("alter table %s alter column %s type %s" % (tableName, columnName, opt[0]))
        self.db.execute("alter table %s alter column %s type %s" % (tableName, columnName, opt[0]))
        return True



    def ConvertConfToColumnOptions(self, conf):
        """
        field representation:
        {"id": "type", "datatype": "list", "size"/"maxLen": 0, "default": ""}

        datatypes:
        list -> list || listn

        string -> VARCHAR(size) NOT NULL DEFAULT default
        number -> INT NOT NULL DEFAULT default
        float -> NUMERIC(16,6) NOT NULL DEFAULT default
        bool -> SMALLINT(4) NOT NULL DEFAULT default
        percent -> SMALLINT(4) NOT NULL DEFAULT default
        text -> TEXT NOT NULL DEFAULT default
        htext -> TEXT NOT NULL DEFAULT default
        lines -> TEXT NOT NULL DEFAULT default
        xml -> TEXT NOT NULL DEFAULT default
        unit -> INT NOT NULL DEFAULT default
        unitlist -> VARCHAR(2048) NOT NULL DEFAULT default
        date -> DATE NULL DEFAULT default
        datetime -> TIMESTAMP NULL DEFAULT default
        time -> TIME NULL DEFAULT default
        timestamp -> DOUBLE PRECISION DEFAULT default
        listt -> VARCHAR(30) NOT NULL DEFAULT default
        listn -> SMALLINT NOT NULL DEFAULT default
        multilist, checkbox -> VARCHAR(2048) NOT NULL DEFAULT default
        binary, [file] -> BYTEA
        [bytesize] -> BIGINT(20) NOT NULL DEFAULT default
        url -> TEXT NOT NULL DEFAULT default
        """
        datatype = conf["datatype"]
        col = ""

        # convert datatype list
        if datatype == "list":
            if isinstance(conf["default"], str):
                datatype = "listt"
            else:
                datatype = "listn"

        if datatype in ("string", "email", "password"):
            if conf.get("size", 150) <= 3:
                col = "CHAR(%d) NOT NULL DEFAULT '%s'" % (conf.get("size", conf.get("maxLen",0)), conf["default"])
            else:
                col = "VARCHAR(%d) NOT NULL DEFAULT '%s'" % (conf.get("size", conf.get("maxLen",0)), conf["default"])

        elif datatype in ("number", "long", "int"):
            cval = conf["default"]
            if cval in ("", " ", None):
                cval = 0
            if isinstance(cval, str):
                cval = int(cval)
            size = conf.get("size", 4)
            if size == 2:
                col = "SMALLINT NOT NULL DEFAULT %d" % (cval)
            elif size == 8:
                col = "BIGINT NOT NULL DEFAULT %d" % (cval)
            else:
                col = "INTEGER NOT NULL DEFAULT %d" % (cval)

        elif datatype == "float":
            cval = conf["default"]
            if cval in ("", " ", None):
                cval = 0
            if isinstance(cval, str):
                cval = float(cval)
            col = "NUMERIC(16,6) NOT NULL DEFAULT %d" % (cval)

        elif datatype == "bool":
            cval = conf["default"]
            if cval in ("", " ", None, "False", "0"):
                cval = 0
            if cval in ("True", "1"):
                cval = 1
            if isinstance(cval, str):
                cval = int(cval)
            col = "SMALLINT NOT NULL DEFAULT %d" % (cval)

        elif datatype in ("text", "htext", "url", "urllist", "json", "code", "lines"):
            col = "TEXT NOT NULL DEFAULT '%s'" % (conf["default"])

        elif datatype == "unit":
            cval = conf["default"]
            if cval in ("", " ", None):
                cval = 0
            if isinstance(cval, str):
                cval = int(cval)
            col = "INTEGER NOT NULL DEFAULT %d" % (cval)

        elif datatype == "unitlist":
            col = "VARCHAR(%d) NOT NULL DEFAULT '%s'" % (conf.get("size", conf.get("maxLen",0)), conf["default"])

        elif datatype == "date":
            cval = conf["default"]
            if cval == () or cval == "()":
                cval = "NULL"
            if cval in ("now", "nowdate", "nowtime"):
                cval = ""
            if isinstance(cval, str) and not cval in ("NOW","NULL"):
                cval = self.ConvertDate(cval)
            if not cval:
                col = "DATE NULL"
            else:
                col = "DATE NULL DEFAULT '%s'" % (cval)

        elif datatype == "datetime":
            cval = conf["default"]
            if cval == () or cval == "()":
                cval = "NULL"
            if cval in ("now", "nowdate", "nowtime"):
                cval = ""
            if isinstance(cval, str) and not cval in ("NOW","NULL"):
                cval = self.ConvertDate(cval)
            if not cval:
                col = "TIMESTAMP NULL"
            else:
                col = "TIMESTAMP NULL DEFAULT '%s'" % (cval)

        elif datatype == "time":
            cval = conf["default"]
            if cval == () or cval == "()":
                cval = "NULL"
            if cval in ("now", "nowtime"):
                cval = ""
            if isinstance(cval, str) and not cval in ("NOW","NULL"):
                cval = self.ConvertDate(cval)
            if cval == "":
                col = "TIME NULL"
            else:
                col = "TIME NULL DEFAULT '%s'" % (cval)

        elif datatype == "timestamp":
            cval = conf["default"]
            col = "NUMERIC(16, 6) DEFAULT %s" % (cval)
            #col = "TIMESTAMP without time zone DEFAULT %s" % (cval)

        elif datatype == "listt":
            col = "VARCHAR(%d) NOT NULL DEFAULT '%s'" % (conf.get("size", conf.get("maxLen",0)), conf["default"])

        elif datatype in ("listn", "codelist"):
            cval = conf["default"]
            if cval in ("", " ", None):
                cval = 0
            if isinstance(cval, str):
                cval = int(cval)
            col = "SMALLINT NOT NULL DEFAULT %d" % (cval)

        elif datatype in ("multilist", "checkbox", "mselection", "mcheckboxes", "radio"):
            col = "VARCHAR(%d) NOT NULL DEFAULT '%s'" % (conf.get("size", conf.get("maxLen",0)), conf["default"])

        elif datatype in ("binary", "file"):
            col = "BYTEA"

        elif datatype == "bytesize":
            cval = conf["default"]
            if cval in ("", " ", None):
                cval = 0
            if isinstance(cval, str):
                cval = int(cval)
            col = "INTEGER NOT NULL DEFAULT %d" % (cval)

        if conf.get("unique"):
            col += " UNIQUE"
        return col



    # physical database structure ----------------------------------------------------------------

    def GetDatabases(self):
        if not self.IsDB():
            return []
        sql = """SELECT datname FROM pg_database;"""
        self.db.execute(sql)
        return self.db.fetchall()


    def GetTables(self):
        if not self.IsDB():
            return []
        sql = """SELECT table_name FROM information_schema.tables WHERE table_schema = 'public';"""
        self.db.execute(sql)
        return self.db.fetchall()


    def GetColumns(self, tableName, structure=None):
        """
        returns a dict of columns. each column is represented by and stored und column id:
        {db: {id, type, identity, default, null}, conf: FieldConf()}
        """
        if not self.IsDB():
            return []
        if not self.IsTable(tableName):
            return []
        self.db.execute("SELECT column_name, data_type, column_default, is_nullable, is_identity, character_maximum_length FROM information_schema.columns WHERE table_name ='%s';" % (tableName))
        table = {}
        for c in self.db.fetchall():
            if c[2]:
                default = c[2].replace("::character varying","")
            else:
                default = ""
            if c[5]:
                size = " (%s)"%str(c[5])
            else:
                size = ""
            table[c[0]] = {"db": {"id": c[0], "type": c[1]+size, "identity": c[4], "default": default, "null": c[3]}, "conf": None} 
        if structure:
            for conf in structure:
                if conf.id in table:
                    table[conf.id]["conf"] = conf
                else:
                    table[conf.id] = {"db": None, "conf": conf} 
        return table


    # Tables --------------------------------------------------------------
    """
    inTableEntrys --> ColumnName ColumnDataTyp ColumnOptions
    ColumnOptions --> [NOT][NULL][DEFAULT = ""][PRIMARY KEY][UNIQUE]
    """

    def IsTable(self, tableName):
        for cval in self.GetTables():
            if cval[0].lower() == tableName.lower():
                return True
        return False


    def CreateTable(self, tableName, columns=None, createIdentity=True, primaryKeyName="id"):
        if not self.IsDB():
            return False
        if not tableName:
            return False
        assert(tableName != "user")
        if createIdentity:
            sql = "CREATE TABLE %s(%s SERIAL PRIMARY KEY" % (tableName, primaryKeyName)
            if columns:
                for c in columns:
                    if c.id == primaryKeyName:
                        continue
                    sql += ","
                    sql += c.id + " " + self.ConvertConfToColumnOptions(c)
            sql += ")"
        else:
            sql = "CREATE TABLE %s" % (tableName)
            if not columns:
                raise ConfigurationError("No database fields defined.")
            aCnt = 0
            sql += "("
            for c in columns:
                if aCnt:
                    sql += ","
                aCnt = 1
                sql += c.id + " " + self.ConvertConfToColumnOptions(c)
            sql += ")"
        self.log.debug(sql)
        self.db.execute(sql)

        # delay until table is created
        time.sleep(0.3)
        aCnt = 1
        while not self.IsTable(tableName):
            time.sleep(0.5)
            aCnt += 1
            if aCnt == 30:
                return False
        return True

    def RenameTable(self, inTableName, inNewTableName):
        if not self.IsDB():
            return False
        self.log.debug("alter table %s rename to %s" % (inTableName, inNewTableName))
        self.db.execute("alter table %s rename to %s" % (inTableName, inNewTableName))
        return True



    # Columns --------------------------------------------------------------
    """
    inColumnData --> "ColumnName ColumnDatdatatype ColumnOptions"
        Column Options --> [Not] [NULL] [DEFAULT 'default']
    SetPrimaryKey&SetUnique --> can also get a ColumnList
        ColumnList --> ColumnName1,ColumName2,...
    """

    def IsColumn(self, tableName, columnName):
        columns = self.GetColumns(tableName)
        cn = columnName.lower()
        if cn in columns and columns[cn].get("db"):
            return True
        return False


    def CreateColumn(self, tableName, columnName, columnOptions=""):
        if not self.IsDB():
            return False
        if not columnName or not tableName:
            return False
        if columnOptions == "identity":
            columnOptions = "SERIAL PRIMARY KEY"
        self.log.debug("alter table %s add column %s %s" % (tableName, columnName, columnOptions))
        self.db.execute("alter table %s add column %s %s" % (tableName, columnName, columnOptions))
        return True


    def CreateIdentityColumn(self, tableName, columnName):
        if not self.IsDB():
            return False
        return self.CreateColumn(tableName, columnName, "SERIAL PRIMARY KEY")





# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
# Released under GPL3. See license.txt
#

__doc__ = """
This file provides *container* functionality for :term:`objects` and :term:`roots`. The different components
are separated into classes by functionality. 

"""

from datetime import datetime

from nive.definitions import implementer
from nive.definitions import Conf, baseConf
from nive.definitions import StagContainer
from nive.definitions import IContainer, IRoot, ICache, IObject, IConf, IObjectConf
from nive.definitions import ContainmentError, ConfigurationError, PermissionError
from nive.definitions import AllTypesAllowed
from nive.security import SetupRuntimeAcls
from nive.workflow import WorkflowNotAllowed, ObjectWorkflow, RootWorkflow
from nive.helper import ResolveName, ClassFactory, GetVirtualObj
from nive.i18n import translate

from nive.events import Events
from nive.search import Search
from nive.objects import Object


class ContainerRead(Events):
    """
    Container implementation with read access for subobjects used for objects and roots.

    Requires: ContainerFactory
    """
    defaultSort = "id"

    def obj(self, id, **kw):
        """ shortcut for GetObj """
        return self.GetObj(id, **kw)

    def __getitem__(self, id):
        o = self.GetObj(id)
        if o:
            return o
        raise KeyError(id)

    # Container functions ----------------------------------------------------

    def GetObj(self, id, **kw):
        """
        Get the subobject by id. ::

            id = object id as number
            permission = check current users permissions for the page. requires securityContext.
            securityContext = required to check the permission. the request object by default.
            **kw = load information
            returns the object or None

        Events:
        - loadObj(obj)
        """
        try:
            id = int(id)
        except:
            return None
        if not id:
            return None
        obj = self.factory.DbObj(id, **kw)
        self.Signal("loadObj", obj)
        return obj

    def GetObjs(self, parameter=None, operators=None, pool_type=None, containerOnly=False, **kw):
        """
        Search for subobjects based on parameter and operators. ::

            parameter = dict. see nive.Search
            operators = dict. see nive.Search
            kw.sort = sort objects. if None container default sort is used
            kw.batch = load subobjects as batch
            **kw = see Container.GetObj()
            returns all matching subobjects as list

        see :class:`nive.Search` for parameter/operators description

        Events
        - loadObj(obj)
        """
        if parameter is None:
            parameter = {}
        if operators is None:
            operators = {}

        if containerOnly:
            parameter["pool_stag"] = (StagContainer, StagContainer + 9)
            operators["pool_stag"] = "BETWEEN"

        parameter["pool_unitref"] = self.id
        sort = kw.get("sort", self.defaultSort)
        fields = ["id", "pool_datatbl", "pool_dataref", "pool_type"]
        root = self.root
        if kw.get("queryRestraints") != False:
            parameter, operators = root.ObjQueryRestraints(self, parameter, operators)
        objects = root.search.SelectDict(pool_type=pool_type, parameter=parameter, fields=fields, operators=operators,
                                         sort=sort)
        useBatch = kw.get("batch", True)
        if useBatch:
            ids = [c["id"] for c in objects]
            kw["meta"] = objects
            kw["sort"] = sort
            objs = self.factory.ObjBatch(ids, queryRestraint=False, **kw)
            return objs
        objs = []
        for c in objects:
            try:
                kw["pool_datatbl"] = c["pool_datatbl"]
                kw["pool_dataref"] = c["pool_dataref"]
                try:
                    obj = self.factory.DbObj(c["id"], queryRestraints=False, **kw)
                except PermissionError:
                    # ignore permission errors and continue
                    obj = None
                if obj:
                    objs.append(obj)
            except:
                pass
        self.Signal("loadObj", objs)
        return objs

    def GetObjsList(self, fields=None, parameter=None, operators=None, pool_type=None, containerOnly=False, **kw):
        """
        Search for subobjects based on parameter and operators. This function performs a sql query based on parameters and
        does not load any object. ::

            fields = list. see nive.Search
            parameter = dict. see nive.Search
            operators = dict. see nive.Search
            kw.sort = sort objects. if None container default sort is used
            kw.batch = load subobjects as batch
            returns dictionary list

        see :class:`nive.Search` for parameter/operators description
        """
        if parameter is None:
            parameter = {}
        if operators is None:
            operators = {}

        if containerOnly:
            parameter["pool_stag"] = (StagContainer, StagContainer + 9)
            operators["pool_stag"] = "BETWEEN"

        parameter["pool_unitref"] = self.id
        sort = kw.get("sort", self.defaultSort)
        if not fields:
            # lookup meta list default fields
            fields = self.app.configuration.listDefault
        root = self.root
        parameter, operators = root.ObjQueryRestraints(self, parameter, operators)
        objects = root.search.SelectDict(pool_type=pool_type, parameter=parameter, fields=fields, operators=operators,
                                         sort=sort)
        return objects

    def GetObjsBatch(self, ids, **kw):
        """
        Tries to load the objects with as few sql queries as possible. ::

            ids = list of object ids as number
            **kw = see Container.GetObj()
            returns all matching sub objects as list

        Events
        - loadObj(objs)
        """
        sort = kw.get("sort", self.defaultSort)
        fields = ["id", "pool_datatbl", "pool_dataref"]
        parameter, operators = {}, {}
        parameter["id"] = ids
        parameter["pool_unitref"] = self.id
        operators["id"] = "IN"
        root = self.root
        parameter, operators = root.ObjQueryRestraints(self, parameter, operators)
        objects = root.search.SelectDict(parameter=parameter, fields=fields, operators=operators, sort=sort)
        ids = [c["id"] for c in objects]
        kw["meta"] = objects
        objs = self.factory.ObjBatch(ids, **kw)
        self.Signal("loadObj", objs)
        return objs

    def GetSubtreeIDs(self, sort=None):
        """
        Returns the ids of all contained objects including subfolders sorted by *sort*. Default is *self.defaultSort*.

        returns list
        """
        if not sort:
            sort = self.defaultSort
        db = self.app.db
        ids = db.GetContainedIDs(sort=sort, base=self.id)
        return ids

    def IsContainer(self):
        """ """
        return IContainer.providedBy(self)



    # bw [3]

    def GetSort(self):
        return self.defaultSort

    def GetContainers(self, parameter=None, operators=None, **kw):
        if parameter is None:
            parameter = {}
        if operators is None:
            operators = {}
        parameter["pool_stag"] = (StagContainer, StagContainer + 9)
        operators["pool_stag"] = "BETWEEN"
        objs = self.GetObjs(parameter, operators, **kw)
        return objs

    def GetContainerList(self, fields=None, parameter=None, operators=None, **kw):
        if parameter is None:
            parameter = {}
        if operators is None:
            operators = {}
        parameter["pool_stag"] = (StagContainer, StagContainer + 9)
        operators["pool_stag"] = "BETWEEN"
        return self.GetObjsList(fields, parameter, operators, **kw)

    def GetContainedIDs(self, sort=None):
        return self.GetSubtreeIDs(sort)


class ContainerWrite:
    """
    Container with add and delete functionality for subobjects.

    Provides functionality to allow or disallow the creation of objects based
    on object configuration.subtypes.

    Requires: Container
    """

    def Create(self, type, data, user, **kw):
        """
        Creates a new sub object. ::
        
            type = object type id as string or object configuration
            data = dictionary containing data for the new object
            user = the currently active user
            **kw = version information
            returns the new object or None

        Keyword options:

        - nocommit: pass `nocommit=True` to skip a database commit after creation.

        Events
        
        - beforeAdd(data=data, type=type, user=user, kw) called for the container
        - create(user=user, kw) called for the new object
        - afterAdd(obj=obj, user=user, kw) called for the container after object has been committed
        
        Workflow actions
        
        - add (called in context of the container)
        - create (called in context of the new object)
        """
        app = self.app
        if not IObjectConf.providedBy(type):
            typedef = app.configurationQuery.GetObjectConf(type)
            if not typedef:
                raise ConfigurationError("Type not found (%s)" % (str(type)))
        else:
            typedef = type

        # allow subobject
        if not self.IsTypeAllowed(typedef, user):
            raise ContainmentError("Add type not allowed here (%s)" % (str(type)))

        wf = self.workflow
        if not wf.WfAllow("add", user=user):
            raise WorkflowNotAllowed("Not allowed in current workflow state (add)")

        self.Signal("beforeAdd", data=data, type=type, user=user, **kw)
        db = app.db
        try:
            dbEntry = db.CreateEntry(pool_datatbl=typedef["dbparam"], user=user, **kw)
            obj = self.factory.DbObj(dbEntry.GetID(), dbEntry = dbEntry, parentObj = self, configuration = typedef, **kw)
            if typedef.events:
                obj.SetupEventsFromConfiguration(typedef.events)
            obj.CreateSelf(data, user=user, **kw)
            wf.WfAction("add", user=user)
            obj.Signal("create", user=user, **kw)
            if not kw.get("nocommit") and app.configuration.autocommit:
                obj.CommitInternal(user=user)
        except Exception as e:
            db.Undo()
            raise 
        self.Signal("afterAdd", obj=obj, user=user, **kw)
        return obj


    def CreateWithoutEventsAndSecurity(self, typedef, data, user, **kw):
        """
        Creates a new sub object. Unlike `Create` this function does not trigger any events
        or workflow actions. Subtype configuration is not validated.
        Also commit is not called. Can be used to speed up batch creation. ::

            typedef = object type configuration
            data = dictionary containing data for the new object
            user = the currently active user
            **kw = version information
            returns the new object or None

        """
        app = self.app
        db = app.db
        dbEntry = None
        try:
            dbEntry = db.CreateEntry(pool_datatbl=typedef["dbparam"], user=user, **kw)
            obj = self.factory.DbObj(dbEntry.GetID(), dbEntry = dbEntry, parentObj = self, configuration = typedef, **kw)
            obj.UpdateInternal(data)
            obj.meta["pool_type"] = typedef.id
            obj.meta["pool_unitref"] = obj.parent.id
            obj.meta["pool_stag"] = obj.selectTag
        except Exception as e:
            db.Undo()
            raise
        return obj


    def Duplicate(self, obj, user, updateValues=None, **kw):
        """
        Duplicate the object including all data and files and store as new subobject. ::
        
            obj = the object to be duplicated
            user = the currently active user
            updateValues = dictionary containing meta, data, files
            **kw = version information
            returns new object or None
            
        Events
        
        - beforeAdd(data=data, type=type, user=user, kw) called for the container
        - duplicate(kw) called for the new object
        - afterAdd(obj=obj, user=user, kw) called for the container after obj has been committed

        Workflow action

        - add (called in context of the container)
        - create (called in context of the new object)
        """
        if updateValues is None:
            updateValues = dict()
        app = self.app
        type = obj.GetTypeID()
        # allow subobject
        if not self.IsTypeAllowed(type, user):
            raise ContainmentError("Add type not allowed here (%s)" % (str(type)))
        
        wf = ObjectWorkflow(self)
        if not wf.WfAllow("add", user=user):
            raise WorkflowNotAllowed("Workflow: Not allowed (add)")

        self.Signal("beforeAdd", data=updateValues, type=type, user=user, **kw)
        newDataEntry = None
        try:
            newDataEntry = obj.dbEntry.Duplicate()
            updateValues["pool_unitref"] = self.id
            updateValues["pool_wfa"] = ""
            data, meta, files = obj.SplitData(updateValues)
            newDataEntry.meta.update(meta)
            newDataEntry.data.update(data)
            newDataEntry.files.update(files)
            id = newDataEntry.GetID()
            typedef = obj.configuration
    
            newobj = self.factory.DbObj(id, dbEntry = newDataEntry, parentObj = self, configuration = typedef)
            newobj.CreateSelf(data, user=user)
            newobj.Signal("duplicate", **kw)
        except Exception as e:
            db = app.db
            if newDataEntry:
                try:
                    id = newDataEntry.GetID()
                    db.DeleteEntry(id)
                except:
                    pass
            db.Undo()
            raise 

        try:    
            if obj.IsContainer():
                for o in obj.GetObjs(queryRestraints=False):
                    newobj._RecursiveDuplicate(o, user, **kw)
            wf.WfAction("add", user=user)
            if app.configuration.autocommit:
                newobj.CommitInternal(user=user)
        except Exception as e:
             self._DeleteObj(newobj)
             raise 
         
        self.Signal("afterAdd", obj=newobj, user=user, **kw)
        return newobj
    
        
    def _RecursiveDuplicate(self, obj, user, **kw):
        """
        Recursively duplicate all subobjects. 
        """
        type=obj.GetTypeID()
        
        updateValues = {}
        self.Signal("beforeAdd", data=updateValues, type=type, **kw)
        newDataEntry = obj.dbEntry.Duplicate()
        updateValues["pool_unitref"] = self.id
        updateValues["pool_wfa"] = ""
        data, meta, files = obj.SplitData(updateValues)
        newDataEntry.meta.update(meta)
        newDataEntry.data.update(data)
        newDataEntry.files.update(files)
        id = newDataEntry.GetID()
        typedef = obj.configuration

        newobj = self.factory.DbObj(id, dbEntry = newDataEntry, parentObj = self, configuration = typedef)
        newobj.CreateSelf({}, user=user)
        newobj.Signal("duplicate", **kw)
        
        if obj.IsContainer():
            for o in obj.GetObjs(queryRestraints=False):
                newobj._RecursiveDuplicate(o, user, **kw)

        if self.app.configuration.autocommit:
            newobj.CommitInternal(user=user)

        self.Signal("afterAdd", obj=newobj, **kw)


    def Delete(self, id, user, obj=None, **kw):
        """
        Delete the subobject referenced by id. ::
        
            id = id of object to be deleted
            user = the currently active user
            obj = the object to be deleted. Will be loaded automatically if None
            **kw = version information
            returns True or False
        
        Events
        
        - delete(user=user) called on object to be deleted
        - afterDelete(id=id, user=user) called on container after object has been deleted

        Workflow action

        - remove (called in context of the container)
        - delete (called in context of the new object)
        """
        app = self.app
        # check if id is object
        if IObject.providedBy(id):
            obj = id
        if obj is None:
            obj = self.GetObj(id, queryRestraints=False, **kw)
            if obj is None:
                return False
        if obj.parent.id != self.id:
            raise ContainmentError("Object is not a child (%s)" % (str(id)))

        # call workflow
        wf = ObjectWorkflow(self)
        if not wf.WfAllow("remove", user=user):
            raise WorkflowNotAllowed("Workflow: Not allowed (remove)")
        if not obj.workflow.WfAllow("delete", user=user):
            raise WorkflowNotAllowed("Workflow: Not allowed (delete)")
        obj.Signal("delete", user=user)
        if hasattr(obj, "_RecursiveDelete"):
            obj._RecursiveDelete(user)

        # call workflow
        obj.workflow.WfAction("delete", user=user)
        self._DeleteObj(obj)
        if app.configuration.autocommit:
            self.db.Commit()
        wf.WfAction("remove", user=user)
        self.Signal("afterDelete", id=id, user=user)

        return True


    def DeleteInternal(self, id, user, obj=None, **kw):
        """
        Like *Delete()* but does not call any workflow action. ::
        
            id = id of object to be deleted
            user = the currently active user
            obj = the object to be deleted. Will be loaded automatically if None
            **kw = version information
            returns True or False

        Events
        
        - delete() called on object to be deleted
        - afterDelete(id=id) called on container after object has been deleted
        """
        app = self.app
        if not obj:
            obj = self.GetObj(id, queryRestraints=False, **kw)
        else:
            if not obj.parent==self:
                raise ContainmentError("Object is not a child (%s)" % (str(id)))
        obj.Signal("delete")
        if hasattr(obj, "_RecursiveDeleteInternal"):
            obj._RecursiveDeleteInternal(user)
        self._DeleteObj(obj)
        self.Signal("afterDelete", id=id)
        return 1


    def _DeleteObj(self, obj, id=0):
        """
        Deletes the object and additional data from wfdata, wflog, fulltext,
        security tables.
        """
        if obj is not None:
            id = obj.id
        obj.Close()
        if id > 0:
            self.db.DeleteEntry(id)


    def _RecursiveDelete(self, user):
        """
        Recursively deletes all subobjects
        """
        objs = self.GetObjs(queryRestraints=False)
        for o in objs:
            self.Delete(o.id, obj=o, user=user)


    def _RecursiveDeleteInternal(self, user):
        """
        Recursively deletes all subobjects
        """
        objs = self.GetObjs(queryRestraints=False)
        for o in objs:
            self.DeleteInternal(o.id, obj=o, user=user)
            
            
    def GetAllowedTypes(self, user=None, visible=1):
        """
        List types allowed to be created in this container based on
        configuration.subtypes.    ::
        
            user = the currently active user
            visible = will skip all hidden object types 
            returns list of configurations
            
        """
        subtypes = self.configuration.subtypes
        if not subtypes:
            return ()
        all = self.app.configurationQuery.GetAllObjectConfs(visibleOnly=visible)
        if subtypes == AllTypesAllowed:
            return all

        #allids = [c.id for c in all]
        # check types by interface
        allowed = []
        for conf in all:
            # create type from configuration
            type = GetVirtualObj(conf, self.app)
            if not type:
                continue
            # loop subtypes
            for iface in subtypes:
                if isinstance(iface, str):
                    if conf.id==iface:
                        # type id
                        allowed.append(conf)
                        continue
                    iface = ResolveName(iface, raiseExcp=False)
                    if not iface:
                        continue
                try:
                    # may not be interface class
                    if iface.providedBy(type):
                        allowed.append(conf)
                except:
                    pass
        return allowed


    def IsTypeAllowed(self, type, user=None):
        """
        Check if *type* is allowed to be created in this container based on
        configuration.subtypes.::
        
            type = the type to be checked
            user = the currently active user
            returns True/False
        
        *type* can be passed as
        
        - type id string
        - configuration object
        - type object instance
        """
        if type is None:
            return False
        subtypes = self.configuration.subtypes
        if subtypes == AllTypesAllowed:
            return True
        if not subtypes:
            return False

        if isinstance(type, str):
            if type in subtypes:
                return True
            # dotted python to obj configuration
            type = self.app.configurationQuery.GetObjectConf(type)
            if type is None:
                return False
            # create type from configuration
            type = GetVirtualObj(type, self.app)
            if type is None:
                return False

        if isinstance(type, baseConf):
            if type.id in subtypes:
                return True
            # create type from configuration
            type = GetVirtualObj(type, self.app)
            if type is None:
                return False

        if not IObject.providedBy(type) and not IConf.providedBy(type):
            return False
        # loop subtypes
        for iface in subtypes:
            if isinstance(iface, str):
                iface = ResolveName(iface, raiseExcp=False)
                if not iface:
                    continue
            try:
                # may not be interface class
                if iface.providedBy(type):
                    return True
            except:
                pass
        return False



@implementer(IContainer, IObject)
class Container(Object, ContainerRead, ContainerWrite):
    """
    Container implementation for objects and roots.
    """

    @property
    def factory(self):
        return ContainerFactory(self)

    @property
    def workflow(self):
        return ObjectWorkflow(self)



@implementer(IContainer, IRoot)
class Root(ContainerRead, ContainerWrite):
    """
    The root is a container for objects but does not store any data in the database itself. It
    is the entry point for object access. Roots are only handled by the application.

    Requires (Container, ContainerFactory, Event)
    """

    def __init__(self, path, app, rootDef):
        self.__name__ = str(path)
        self.__parent__ = app
        self.id = 0
        # unique root id generated from name . negative integer.
        self.idhash = abs(hash(self.__name__)) * -1
        self.path = path
        self.configuration = rootDef
        self.queryRestraints = {}, {}

        self.meta = Conf(pool_type=rootDef["id"],
                         title=translate(rootDef["name"]),
                         pool_state=1,
                         pool_filename=path,
                         pool_wfa="",
                         pool_change=datetime.now(tz=app.pytimezone),
                         pool_changedby="",
                         pool_create=datetime.now(tz=app.pytimezone),
                         pool_createdby="")
        self.data = Conf()
        self.files = Conf()

        # load security
        acl = SetupRuntimeAcls(rootDef.acl, self)
        if acl:
            # omit if empty. acls will be loaded from parent object
            self.__acl__ = acl

        self.Signal("init")

    # Properties -----------------------------------------------------------

    @property
    def root(self):
        """ this will return itself. Used for object compatibility. """
        return self

    @property
    def app(self):
        """ returns the cms application the root is used for """
        return self.__parent__

    @property
    def db(self):
        """ returns the datapool object """
        return self.app.db

    @property
    def parent(self):
        """ this will return None. Used for object compatibility. """
        return None

    @property
    def factory(self):
        return ContainerFactory(self)

    @property
    def workflow(self):
        return RootWorkflow(self)

    @property
    def search(self):
        return Search(self)

    # Object Lookup -----------------------------------------------------------

    def LookupObj(self, id, **kw):
        """
        Lookup the object referenced by id *anywhere* in the tree structure. Use obj() to
        restrain lookup to the first sublevel only. ::

            id = number
            **kw = version information

        returns the object or None
        """
        try:
            id = int(id)
        except:
            return None
        if id <= 0:
            return self
        if not id:
            return None
            # raise Exception, "NotFound"

        # proxy object
        if "proxyObj" in kw and kw["proxyObj"]:
            obj = self.factory.DbObj(id, parentObj=kw["proxyObj"], **kw)
            if not obj:
                raise ContainmentError("Proxy object not found")
            return obj

        # load tree structure
        path = self.app.db.GetParentPath(id)
        if path is None:
            return None
            # raise Exception, "NotFound"

        # check and adjust root id
        if hasattr(self, "rootID"):
            if self.rootID in path:
                path = path[path.index(self.rootID) + 1:]
        if hasattr(self.app, "rootID"):
            if self.app.rootID in path:
                path = path[path.index(self.app.rootID) + 1:]

        # reverse lookup of object tree. loads all parent objs.
        path.append(id)
        # opt
        obj = self
        for id in path:
            if id == self.id:
                continue
            obj = obj.factory.DbObj(id, **kw)
            if not obj:
                return None
                # raise Exception, "NotFound"
        return obj

    def ObjQueryRestraints(self, containerObj=None, parameter=None, operators=None):
        """
        The functions returns two dictionaries (parameter, operators) used to restraint
        object lookup in subtree. For example a restraint can be set to ignore all
        objects with meta.pool_state=0. All container get (GetObj, GetObjs, ...) functions
        use query restraints internally.

        See `nive.search` for parameter and operator usage.

        Please note: Setting the wrong values for query restraints can easily crash
        the application.

        Event:
        - loadRestraints(parameter, operators)

        returns parameter dict, operators dict
        """
        p, o = self.queryRestraints
        if parameter:
            parameter.update(p)
            if operators:
                operators.update(o)
            else:
                operators = o.copy()
        else:
            parameter = p.copy()
            operators = o.copy()
        self.Signal("loadRestraints", parameter=parameter, operators=operators)
        return parameter, operators

    # Values ------------------------------------------------------

    def GetID(self):
        """ returns 0. the root id is always zero. """
        return self.id

    def GetTypeID(self):
        """ returns the root type id from configuration """
        return self.configuration.id

    def GetTypeName(self):
        """ returns the root type name from configuration """
        return self.configuration.name

    def GetFieldConf(self, fldId):
        """
        Get the FieldConf for the field with id = fldId. Looks up data, file and meta
        fields.

        returns FieldConf or None
        """
        for f in self.configuration["data"]:
            if f["id"] == fldId:
                return f
        return self.app.configurationQuery.GetMetaFld(fldId)

    def GetTitle(self):
        """ returns the root title from configuration. """
        return self.meta.get("title", "")

    def GetPath(self):
        """ returns the url path name as string. """
        return self.__name__

    # Parents ----------------------------------------------------

    def IsRoot(self):
        """ returns always True. """
        return True

    def GetParents(self):
        """ returns empty list. Used for object compatibility. """
        return ()

    def GetParentIDs(self):
        """ returns empty list. Used for object compatibility. """
        return ()

    def GetParentTitles(self):
        """ returns empty list. Used for object compatibility. """
        return ()

    def GetParentPaths(self):
        """ returns empty list. Used for object compatibility. """
        return ()

    # tools ----------------------------------------------------

    def GetTool(self, name):
        """
        Load a tool in the roots' context. Only works for tools registered for roots or this root type. ::

            returns the tool object or None

        Event
        - loadToool(tool=toolObj)
        """
        t = self.app.GetTool(name, self)
        self.Signal("loadTool", tool=t)
        return t

    def Close(self):
        """
        Close the root and all contained objects. Currently only used in combination with caches.

        Event
        - close()
        """
        self.Signal("close")
        if ICache.providedBy(self):
            # opt
            for o in self.GetAllFromCache():
                o.Close()
        return


class ContainerFactory(object):
    """
    Container object factory. Creates objects based on type configuration.
    """

    def __init__(self, obj):
        self.obj = obj



    def DbObj(self, id, dbEntry=None, parentObj=None, configuration=None, **kw):
        """
        Loads and initializes the object. ::

            id = id of the object to be loaded
            dbEntry = the database entry. Will be loaded automatically if None
            parentObj = if a different parent than the container
            configuration = the object configuration to be loaded
            returns the object or None

        """
        obj = self.obj
        useCache = ICache.providedBy(obj)
        if useCache:
            o = obj.GetFromCache(id)
            if o:
                return o
        app = obj.app
        if not dbEntry:
            # check restraints
            qr = kw.get("queryRestraints", None)
            if qr != False:
                root = obj.root
                p, o = root.ObjQueryRestraints(obj)
                p["id"] = id
                p["pool_unitref"] = obj.id
                e = root.search.Select(parameter=p, operators=o)
                if len(e) == 0:
                    return None

            dbEntry = app.db.GetEntry(id, **kw)
            if not dbEntry:
                return None
                # raise Exception, "NotFound"

            if dbEntry.meta["pool_unitref"] != obj.id:
                raise ContainmentError("Object is not a child (%s)" % (str(id)))

        # create object for type
        if not parentObj:
            parentObj = obj
        if not configuration:
            type = dbEntry.GetMetaField("pool_type")
            if not type:
                # broken entry
                # raise ConfigurationError, "Empty type"
                return None
            configuration = app.configurationQuery.GetObjectConf(type)
            if not configuration:
                raise ConfigurationError("Type not found (%s)" % (str(type)))
        newobj = ClassFactory(configuration, app.reloadExtensions, True, base=None)
        newobj = newobj(id, dbEntry, parent=parentObj, configuration=configuration, **kw)

        # check security if context passed in keywords
        if kw.get("securityContext") is not None and kw.get("permission"):
            if not kw["securityContext"].has_permission(kw["permission"], newobj):
                raise PermissionError("Permission check failed (%s)" % (str(id)))

        if useCache:
            obj.Cache(newobj, newobj.id)
        return newobj

    def ObjBatch(self, ids, parentObj=None, **kw):
        """
        Load multiple objects at once.
        """
        if len(ids) == 0:
            return []
        obj = self.obj
        useCache = ICache.providedBy(self.obj)
        objs = []
        if useCache:
            load = []
            for id in ids:
                o = obj.GetFromCache(id)
                if o:
                    objs.append(o)
                else:
                    load.append(id)
            if len(load) == 0:
                return objs
            ids = load

        app = obj.app
        entries = app.db.GetBatch(ids, preload="all", **kw)

        # create object for type
        if not parentObj:
            parentObj = obj
        securityContext = kw.get("securityContext")
        permission = kw.get("permission")
        for dbEntry in entries:
            type = dbEntry.meta.get("pool_type")
            if not type:
                continue
            configuration = app.configurationQuery.GetObjectConf(type, skipRoot=1)
            if not configuration:
                continue
            newobj = ClassFactory(configuration, app.reloadExtensions, True, base=None)
            newobj = newobj(dbEntry.id, dbEntry, parent=parentObj, configuration=configuration, **kw)

            # check security if context passed in keywords
            if securityContext is not None and permission:
                if not securityContext.has_permission(permission, obj):
                    continue

            if useCache:
                obj.Cache(newobj, newobj.id)
            objs.append(newobj)
        return objs


    def VirtualObj(self, configuration):
        """
        This loads an object for a non existing database entry.
        """
        if configuration is None:
            raise ConfigurationError("Type not found")
        app = self.obj.app
        obj = ClassFactory(configuration, app.reloadExtensions, True, base=None)
        dbEntry = app.db.GetEntry(0, virtual=1)
        obj = obj(0, dbEntry, parent=None, configuration=configuration)
        return obj



import unittest

class TestAPI(unittest.TestCase):
    def test_it(self):
        # none of these imports should fail
        from nive.components.reform import Form
        from nive.components.reform import Button
        from nive.components.reform import Field

        from nive.components.reform import FileData
        from nive.components.reform import Set

        from nive.components.reform import ValidationFailure
        from nive.components.reform import TemplateError

        from nive.components.reform import ZPTRendererFactory
        from nive.components.reform import default_renderer

        from nive.components.reform import widget
        
        
        
        
        





try:
    from PyRSS2Gen import PyRSS2Gen
except:
    pass
from datetime import datetime

from nive.definitions import Conf, ViewConf
from nive.views import BaseView


class RSSView(BaseView):

    def rss(self):
        """
        default rss publishing function
        """
        feedData = dict(link=self.Url())
        feed = self.context.RssSearch(feedData)
        return self.SendResponse(feed, mime="application/rss+xml")


    def RssSearch(self, feedData, view=None):
        """
        generate the folder contents as rss feed

        opotions:
        rssTypes = list. default: all non-folders. list of pool_type to be published
        count = number. default:30. number of entries
        condition = default none. select condition.
        operators = default none.
        itemFieldMap = dictionary. default:{"title": meta.title, "link": item.GetURL(), "description": data.description, "pubDate": pool_change}.
                       a mapping table of fields to generate the feeds.
        includeSubfolder = bool. default: true

        add in tmpl header: <link rel="alternate" type="application/rss+xml"  href="rss" title="title">


        :param feedData:
        :param view: required if item link is not set
        :return:
        """

        conf = self.context.configuration.rss
        root = self.context.root

        parameter = conf.parameter.copy()
        if conf.container and not "pool_unitref" in parameter:
            parameter["pool_unitref"] = self.context.id

        # search
        if not conf.pool_type:
            items = root.search.Search(parameter, fields=conf.fields, operators=conf.operators, sort=conf.sort, max=conf.count)
        else:
            items = root.search.SearchType(conf.pool_type, parameter, fields=conf.fields, operators=conf.operators, sort=conf.sort, max=conf.count)

        return self.RssXml(items, feedData, conf, view)


    def RssXml(self, items, feedData, conf, view):
        """

        :param items:
        :param feedData:
        :param conf:
        :param view:
        :return:
        """
        context = self.context
        if not "title" in feedData:
            feedData["title" ] = context.meta.title
        if not "description" in feedData:
            feedData["description" ] = context.data.get("description") or conf.description
        if not "inputEncoding" in feedData and conf.inputEncoding:
            feedData["inputEncoding" ] = conf.inputEncoding

        feed = RssWriter(**feedData)
        itemFieldMap = conf.itemFieldMap
        root = self.context.root
        for i in items["items"]:
            item = {}
            if "link" in itemFieldMap:
                item["link"] = i[itemFieldMap["link"]]
            else:
                item["link"] = view.Url(root.LookupObj(i["id"], preload="skip"))
            item["guid"] = item["link"]
            if "pubDate" in itemFieldMap:
                item["pubDate"] = renderGMT(i[itemFieldMap["pubDate"]], "0200")
            for k in itemFieldMap:
                if k in ("link", "pubDate"):
                    continue
                item[k] = i[itemFieldMap[k]]

            feed.AddItem(**item)

        data = feed.Publish(feedData)
        data = data.replace("&lt;", "<")
        data = data.replace("&gt;", ">")
        return data



class RssWriter(object):
    """
    Generates rss 2 feeds from dictionary lists (Uses PyRSS2Gen module)

    Feed:
        title = "The title of the feed"
        link = URL
        description = "long description of the feed"
        lastBuildDate = default: now

      extended:
        language = None
        copyright = None
        managingEditor = None
        webMaster = None
        pubDate = None  		# a datetime, *in* *GMT*

        categories = None 		# list of strings or Category
        generator = _generator_name
        docs = "http://blogs.law.harvard.edu/tech/rss"
        cloud = (domain, port, path, registerProcedure, protocol) default:None			# a Cloud
        ttl = None	  			# integer number of minutes

        image = (url, title, link, width = None, height = None, description = None) default:None			# an Image
        rating = string default:None			# a string; I don't know how it's used
        textInput = (title, description, name, link) default:None 		# a special element TextInput
        skipHours = [12,13,14] default:None 		# a SkipHours with a list of integers
        skipDays = ["Sat", "Sun"] default:None  		# a SkipDays with a list of strings
        inputEncoding = default: utf-8

    Items:
        title = "The title of the item"
        link = URL
        description = "long description of the item"
        guid = (baseUrl, itemPath)
        pubDate = default: now

      extended:
        author = None	  	# email address as string
        categories = None  	# list of string or Category
        comments = None  	# url as string
        enclosure = (url, length, type) default:None 	# an Enclosure
        guid = None			# a unique string
        source = None  		# a Source
    """

    _generator_name = "rss.2"
    _input_encoding = "utf-8"
    _xmlns = {"xmlns:atom": "http://www.w3.org/2005/Atom"}


    def __init__(self, title, link, description, lastBuildDate = None, **kw):
        self.Set(title, link, description, lastBuildDate = None, **kw)


    def Set(self, title, link, description, lastBuildDate = None, **kw):
        """
        set feed content
        """
        self.inputEncoding = kw.get("inputEncoding", self._input_encoding)
        if not lastBuildDate:
            lastBuildDate = datetime.now()

        categories = kw.get("categories")
        if categories:
            if not isinstance(categories, (list, tuple)):
                categories = PyRSS2Gen.Category(*categories)
        cloud = kw.get("cloud")
        if cloud:
            cloud = PyRSS2Gen.Cloud(*cloud)
        image = kw.get("image")
        if image:
            image = PyRSS2Gen.Image(*image)
        textInput = kw.get("textInput")
        if textInput:
            textInput = PyRSS2Gen.TextInput(*textInput)
        skipHours = kw.get("skipHours")
        if skipHours:
            skipHours = PyRSS2Gen.SkipHours(*skipHours)
        skipDays = kw.get("skipDays")
        if skipDays:
            skipDays = PyRSS2Gen.SkipDays(*skipDays)

        self.feed = PyRSS2Gen.RSS2(title,
                                   link,
                                   description,

                                   language = kw.get("language"),
                                   copyright = kw.get("copyright"),
                                   managingEditor = kw.get("managingEditor"),
                                   webMaster = kw.get("webMaster"),
                                   pubDate = kw.get("pubDate"),
                                   lastBuildDate = lastBuildDate,

                                   categories = categories,
                                   cloud = cloud,
                                   image = image,
                                   rating = kw.get("rating"),
                                   textInput = textInput,
                                   skipHours = skipHours,
                                   skipDays = skipDays,
                                   ttl = kw.get("ttl"),
                                   inputEncoding = self.inputEncoding,

                                   generator = self._generator_name,
                                   docs = "http://blogs.law.harvard.edu/tech/rss"
                                   )
        self.feed.rss_attrs.update(self._xmlns)


    def AddItem(self, title, link, **kw):
        """
        add an item to the feed
        """
        categories = kw.get("categories")
        if categories:
            if type(categories) == type(""):
                categories = [categories]
        #categories = PyRSS2Gen.Category(*categories)

        enclosure = kw.get("enclosure")
        #if enclosure:
        #	if type(enclosure) == type(""):
        #		enclosure = [enclosure]
        #	enclosure = PyRSS2Gen.Enclosure(*enclosure)

        guid = kw.get("guid")
        #if guid:
        #	if type(guid) == type(""):
        #		guid = [guid]
        #	guid = PyRSS2Gen.Guid(*guid)

        source = kw.get("source")
        #if source:
        #	if type(source) == type(""):
        #		source = [source]
        #	source = PyRSS2Gen.Source(*source)
        content = kw.get("content")

        item = PyRSS2Gen.RSSItem(title,
                                 link,
                                 description = kw.get("description"),
                                 author = kw.get("author"),
                                 categories = categories,
                                 comments = kw.get("comments"),
                                 enclosure = enclosure,
                                 guid = guid,
                                 pubDate = kw.get("pubDate"),
                                 source = source,
                                 content = content,
                                 inputEncoding = self.inputEncoding
                                 )

        self.feed.items.append(item)


    def Publish(self, encoding="utf-8"):
        """
        Publish RSS stream. returns stream as string
        """
        return self.feed.to_xml(encoding)


    def PublishFile(self, filename, encoding="utf-8"):
        """
        Publish RSS stream as file.
        """
        with open(filename, "rw") as file:
            self.feed.write_xml(file, encoding)
            file.close()
        return



def renderGMT(dt, offset=None):
    if offset:
        offset = "+" + offset
    return "%s, %02d %s %04d %02d:%02d:%02d %s" % (
        ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"][dt.weekday()],
        dt.day,
        ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"][dt.month - 1],
        dt.year, dt.hour, dt.min, dt.sec, offset)



# include as configuration.rss in the contexts configuration and fill settings as required
settings = Conf(
    pool_type = None,
    operators = dict(),
    parameter = dict(pool_state=1),
    container = True,
    itemFieldMap = dict(title="title", pubDate="pool_change"),
    sort = "pool_change",
    language = "de",
    author = "",
    inputEncoding = ""
)

view_configuration = ViewConf(
    id = "rss_view_conf",
    attr = "rss",
    name = "rss",
    context = None, # required
    view = RSSView,
    permission = None
)




# -*-coding:utf-8 -*-

import unittest

from nive.extensions.persistence import *
from nive.definitions import Conf
from nive.helper import FormatConfTestFailure

from nive.tests import db_app
from nive.tests import __local

        

class Persistence(unittest.TestCase):
    
    def setUp(self):
        self.app = db_app.app_nodb()

    def tearDown(self):
        self.app.Close()


    def test_1(self):
        p = PersistentConf(self.app, self.app.configuration)
        
        
    def test_2(self):
        LoadStoredConfValues(self.app, None)
        
        
        
class tdbPersistence(__local.DefaultTestCase):
    
    def setUp(self):
        self._loadApp(["nive.extensions.persistence.dbPersistenceConfiguration"])

    def tearDown(self):
        self._closeApp()

    
    def test_conf(self):
        r=dbPersistenceConfiguration.test()
        if not r:
            return
        self.fail(FormatConfTestFailure(r))

        
        
    def test_storedconf(self):
        storage = self.app.NewModule(IModuleConf, "persistence")
        self.assertTrue(storage)
        LoadStoredConfValues(self.app, None)


    def test_load(self):
        storage = self.app.NewModule(IModuleConf, "persistence")
        self.assertTrue(storage)
        storage(self.app, Conf(id="test")).Save({"title":"öäüß", "something": 123})

        values = Conf(id="test")
        storage(self.app, values).Load()
        self.assertTrue(values["something"] == 123)
        self.assertTrue(values["title"] == "öäüß")


# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
# Released under GPL3. See license.txt
#

import string, time

from nive.utils.dataPool2.dbManager import DatabaseManager
from nive.definitions import ConfigurationError

SQLITE3 = 0

try:    
    import sqlite3
    SQLITE3 = 1
except:
    pass

    
    
    
class Sqlite3Manager(DatabaseManager):
    """
    """
    modifyColumns = True #False

    def __init__(self):
        # might not be imported on startup
        import sqlite3
        self.db = None
        self.dbConn = None
        self.useUtf8 = True


    def Connect(self, databaseName = ""):
        self.dbConn = sqlite3.connect(databaseName)
        self.db = self.dbConn.cursor()
        return self.db != None


    # Options ----------------------------------------------------------------

    def ConvertConfToColumnOptions(self, conf):
        """
        field representation:
        {"id": "type", "datatype": "list", "size"/"maxLen": 0, "default": ""}

        datatypes:
        list -> list || listn

        string -> VARCHAR(size) NOT NULL DEFAULT default
        number -> INT NOT NULL DEFAULT default
        float -> FLOAT NOT NULL DEFAULT default
        bool -> TINYINT(4) NOT NULL DEFAULT default
        percent -> TINYINT(4) NOT NULL DEFAULT default
        text -> TEXT NOT NULL DEFAULT default
        htext -> TEXT NOT NULL DEFAULT default
        lines -> TEXT NOT NULL DEFAULT default
        xml -> TEXT NOT NULL DEFAULT default
        unit -> INT NOT NULL DEFAULT default
        unitlist -> VARCHAR(2048) NOT NULL DEFAULT default
        date -> TIMESTAMP NULL DEFAULT default
        datetime -> TIMESTAMP NULL DEFAULT default
        time -> TIMESTAMP NULL DEFAULT default
        timestamp -> TIMESTAMP
        listt -> VARCHAR(30) NOT NULL DEFAULT default
        listn -> SMALLINT NOT NULL DEFAULT default
        multilist, checkbox -> VARCHAR(2048) NOT NULL DEFAULT default
        [file] -> BLOB
        [bytesize] -> BIGINT(20) NOT NULL DEFAULT default
        url -> TEXT NOT NULL DEFAULT default
        """
        datatype = conf["datatype"]
        aStr = ""

        # convert datatype list
        if(datatype == "list"):
            if isinstance(conf["default"], str):
                datatype = "listt"
            else:
                datatype = "listn"

        if datatype in ("string", "email", "password"):
            if conf.get("size", conf.get("maxLen",0)) <= 3:
                aStr = "CHAR(%d) NOT NULL DEFAULT '%s'" % (conf.get("size", conf.get("maxLen",0)), conf["default"])
            else:
                aStr = "VARCHAR(%d) NOT NULL DEFAULT '%s'" % (conf.get("size", conf.get("maxLen",0)), conf["default"])

        elif datatype in ("number", "long", "int"):
            aN = conf["default"]
            if aN == "" or aN == " " or aN == None:
                aN = 0
            if isinstance(aN, str):
                aN = int(aN)
            if conf.get("size", conf.get("maxLen",0)) == 4:
                aStr = "TINYINT NOT NULL DEFAULT %d" % (aN)
            elif conf.get("size", conf.get("maxLen",0)) in (11,12):
                aStr = "INTEGER NOT NULL DEFAULT %d" % (aN)
            else:
                aStr = "INTEGER NOT NULL DEFAULT %d" % (aN)

        elif datatype == "float":
            aN = conf["default"]
            if aN == "" or aN == " " or aN == None:
                aN = 0
            if isinstance(aN, str):
                aN = float(aN)
            aStr = "FLOAT NOT NULL DEFAULT %d" % (aN)

        elif datatype == "bool":
            aN = conf["default"]
            if aN == "" or aN == " " or aN == None or aN == "False":
                aN = 0
            if aN == "True":
                aN = 1
            if isinstance(aN, str):
                aN = int(aN)
            aStr = "TINYINT NOT NULL DEFAULT %d" % (aN)

        elif datatype in ("text", "htext", "url", "urllist", "json", "code"):
            aStr = "TEXT NOT NULL DEFAULT '%s'" % (conf["default"])

        elif datatype == "unit":
            aN = conf["default"]
            if aN == "" or aN == " " or aN == None:
                aN = 0
            if isinstance(aN, str):
                aN = int(aN)
            aStr = "INTEGER NOT NULL DEFAULT %d" % (aN)

        elif datatype == "unitlist":
            aStr = "VARCHAR(%d) NOT NULL DEFAULT '%s'" % (conf.get("size", conf.get("maxLen",0)), conf["default"])

        elif datatype == "date":
            aD = conf["default"]
            if aD == () or aD == "()":
                aD = "NULL"
            if aD in ("now", "nowdate", "nowtime"):
                aD = ""
            if isinstance(aD, str) and not aD in ("NOW","NULL"):
                aD = self.ConvertDate(aD)
            if aD == "":
                aStr = "TIMESTAMP NULL"
            else:
                aStr = "TIMESTAMP NULL DEFAULT '%s'" % (aD)

        elif datatype == "datetime":
            aD = conf["default"]
            if aD == () or aD == "()":
                aD = "NULL"
            if aD in ("now", "nowdate", "nowtime"):
                aD = ""
            if isinstance(aD, str) and not aD in ("NOW","NULL"):
                aD = self.ConvertDate(aD)
            if aD == "":
                aStr = "TIMESTAMP NULL"
            else:
                aStr = "TIMESTAMP NULL DEFAULT '%s'" % (aD)

        elif datatype == "time":
            aD = conf["default"]
            if aD == () or aD == "()":
                aD = "NULL"
            if aD in ("now", "nowtime"):
                aD = ""
            if isinstance(aD, str) and not aD in ("NOW","NULL"):
                aD = self.ConvertDate(aD)
            if aD == "":
                aStr = "TIMESTAMP NULL"
            else:
                aStr = "TIMESTAMP NULL DEFAULT '%s'" % (aD)

        elif datatype == "timestamp":
            aStr = "TIMESTAMP DEFAULT (datetime('now','localtime'))"

        elif datatype == "listt":
            aStr = "VARCHAR(%d) NOT NULL DEFAULT '%s'" % (conf.get("size", conf.get("maxLen",0)), conf["default"])

        elif datatype in ("listn", "codelist"):
            aN = conf["default"]
            if aN == "" or aN == " " or aN == None:
                aN = 0
            if isinstance(aN, str):
                aN = int(aN)
            aStr = "SMALLINT NOT NULL DEFAULT %d" % (aN)

        elif datatype in ("multilist", "checkbox", "mselection", "mcheckboxes", "radio"):
            aStr = "VARCHAR(%d) NOT NULL DEFAULT '%s'" % (conf.get("size", conf.get("maxLen",0)), conf["default"])

        elif datatype == "file":
            aStr = "BLOB"

        elif datatype == "bytesize":
            aN = conf["default"]
            if aN == "" or aN == " " or aN == None:
                aN = 0
            if isinstance(aN, str):
                aN = int(aN)
            aStr = "INTEGER NOT NULL DEFAULT %d" % (aN)

        if conf.get("unique"):
            aStr += " UNIQUE"
        return aStr



    # physical database structure ----------------------------------------------------------------

    def GetDatabases(self):
        return []


    def GetTables(self):
        if not self.IsDB():
            return []
        sql = """SELECT name FROM 
                   (SELECT * FROM sqlite_master UNION ALL
                    SELECT * FROM sqlite_temp_master)
                WHERE type='table'
                ORDER BY name"""
        self.db.execute(sql)
        return self.db.fetchall()


    def GetColumns(self, tableName, structure=None):
        """
        returns a dict of columns. each column is represented by and stored und column id:
        {db: {id, type, identity, default, null}, conf: FieldConf()}
        """
        if not self.IsDB():
            return []
        if not self.IsTable(tableName):
            return []
        self.db.execute("PRAGMA table_info(%s)" % (tableName))
        table = {}
        for c in self.db.fetchall():
            table[c[1]] = {"db": {"id": c[1], "type": c[2], "identity": c[5], "default": c[4], "null": c[3]}, 
                           "conf": None} 
        if structure:
            for conf in structure:
                if conf.id in table:
                    table[conf.id]["conf"] = conf
                else:
                    table[conf.id] = {"db": None, "conf": conf} 
        return table


    # Tables --------------------------------------------------------------
    """
    inTableEntrys --> ColumnName ColumnDataTyp ColumnOptions
    ColumnOptions --> [NOT][NULL][DEFAULT = ""][PRIMARY KEY][UNIQUE]
    """

    def IsTable(self, tableName):
        for aD in self.GetTables():
            if aD[0].lower() == tableName.lower():
                return True
        return False


    def CreateTable(self, tableName, columns=None, createIdentity=True, primaryKeyName="id"):
        if not self.IsDB():
            return False
        if not tableName or tableName == "":
            return False
        assert(tableName != "user")
        if createIdentity:
            aSql = "CREATE TABLE %s(%s INTEGER PRIMARY KEY AUTOINCREMENT" % (tableName, primaryKeyName)
            if columns:
                for c in columns:
                    if c.id == primaryKeyName:
                        continue
                    aSql += ","
                    aSql += c.id + " " + self.ConvertConfToColumnOptions(c)
            aSql += ")"
        else:
            aSql = "CREATE TABLE %s" % (tableName)
            if not columns:
                raise ConfigurationError("No database fields defined.")
            aCnt = 0
            aSql += "("
            for c in columns:
                if aCnt:
                    aSql += ","
                aCnt = 1
                aSql += c.id + " " + self.ConvertConfToColumnOptions(c)
            aSql += ")"
        self.db.execute(aSql)
        # delay until table is created
        time.sleep(0.3)
        aCnt = 1
        while not self.IsTable(tableName):
            time.sleep(0.5)
            aCnt += 1
            if aCnt == 30:
                return False
        return True

    def RenameTable(self, inTableName, inNewTableName):
        if not self.IsDB():
            return False
        self.db.execute("alter table %s rename to %s" % (inTableName, inNewTableName))
        return True



    # Columns --------------------------------------------------------------
    """
    inColumnData --> "ColumnName ColumnDatdatatype ColumnOptions"
        Column Options --> [Not] [NULL] [DEFAULT 'default']
    SetPrimaryKey&SetUnique --> can also get a ColumnList
        ColumnList --> ColumnName1,ColumName2,...
    """

    def CreateColumn(self, tableName, columnName, columnOptions=""):
        if not self.IsDB():
            return False
        if columnName == "" or tableName == "":
            return False
        if columnOptions == "identity":
            return self.CreateIdentityColumn(tableName, columnName)
        self.db.execute("alter table %s add column %s %s" % (tableName, columnName, columnOptions))
        return True


    def CreateIdentityColumn(self, tableName, columnName):
        if not self.IsDB():
            return False
        return self.CreateColumn(tableName, columnName, "INTEGER PRIMARY KEY AUTOINCREMENT")


    # Database Options ------------------------------------------------------------

    def IsDatabase(self, databaseName):
        try:
            db = sqlite3.connect(databaseName)
            db.close()
            return True
        except:
            return False

    def CreateDatabase(self, databaseName):
        return False

    def UseDatabase(self, databaseName):
        return False




# -*- coding: utf-8 -*-


from nive.definitions import Conf
from nive.definitions import IObject, IContainer, IRoot, INonContainer

from nive.definitions import baseConf
from nive.definitions import Interface, implementer
from nive.definitions import ConfigurationError



class IIFaceRoot(Interface):
    """ """


class IIFaceConf(Interface):
    """ """


AllTypeDataFlds = 1
AllTypeMetaFlds = 2
AllTypeFlds = 3


@implementer(IIFaceConf)
class IFaceConf(baseConf):
    """
    peak interface view configuration class
    -----------------------------------------
    configuration attributes:
    
    - id: iface module id (optional)
    - name: iface module name (optional)
    - slogan: iface header slogan
    - sections: header bar links
    - currentsection: the active section. can be set for the whole module.
    - backlink: backlink added to head logo
    - headlink: single link on the right, and left of headoptions.
    - headlinkAnonymous: single link on the right, and left of headoptions.
    - headoptions: header options dropdown on the right.
    - navigation: left navigation
    - path: enable or disable breadcrumbs
    - tabs: object content tabs (default or type specific)
    - settings: settings drop down content specific
    - objectheader: object tab header block (default or type specific)
    - shortcuts: object shortcuts bar (default or type specific)
    - pageinfos: dict with page.id providing additional information for page rendering
    - addflds: fields rendered in add form 
    - editflds: fields rendered in edit form
    - metaflds: field list for meta page view (default or type specific)
    - dataflds: field list for data page view (default or type specific)
    - searchconf: search configurations including form, list fields and options. default or type specific. 
    - defaultviews: default (name="") view redirects based on interfaces.
    
    Configures most parts of the interface for html rendering. Required by view classes, parts and search. 
    
    - container list search parameter fields
    - container list fields
    - container list options (cut-copy-paste, delete)
    - edit fields
    - add fields
    - meta fields
    - object shortcuts (add, duplicate, tools, workflow, ccp, delete)
    - object iface tabs
    - interface navigation type
    - interface display name
    
    Url handling in sections, tabs, navigation:
    Calls ToUrl to map item.id to URL. The prefix of id is used to generate the link
    included in the html. e.g. root.search maps to http://server.com/root/search
    Possible values are
    
    - context
    - parent
    - root
    - pool
    - portal
    
    searchfield rendere callback example: ::

        def renderStateIcon(field, value, context, data):
            return '<img src="/static/images/state%d.gif" title="Active state: %d"/>' % (data, data)

    
    implements: IIFaceConf
    """

    def __init__(self, copyFrom=None, **values):
        self.id = ""
        self.name = ""
        self.slogan = ""
        self.logo = ""
        self.cache = "no-cache"
        self.containercls = "container-fluid"
        
        # id: used as url, name: display name, ref: section reference used to highlight active section
        self.sections = [
            Conf(id="root.base",     name="Overview", ref="b"),
            Conf(id="root.view",     name="Content",  ref=""),
            Conf(id="root.search",   name="Search",   ref="s"),
            Conf(id="root.settings", name="Settings", ref="e")
        ]
        
        # item as string or Conf
        # string: called as class method
        # Conf: id: used as url, name: display name, icon: display icon
        self.backlink = None
        self.headlink = None
        self.headlinkanonymous = None
        self.headoptions = [
            "head_logout", 
            #"head_search"
        ]
        
        self.navigation = [
            Conf(id="root.view", name="Folder", widget="navigation_folder", baseID=0, sort="title")
        ]
        
        self.path = True

        self.tabs = [
            Conf(id="context.viewlist", name="Content",  permission= "iface_view", interfaces=[IContainer]),
            Conf(id="context.view",     name="View",     permission= "iface_view", interfaces=[IObject]),
            Conf(id="context.edit",     name="Edit",     permission= "iface_edit", interfaces=[IObject]),
            Conf(id="context.add",      name="Add",      permission= "iface_add",  interfaces=[IContainer]),
            Conf(id="context.options",  name="Options",  permission= "iface_view", interfaces=[IObject]),
            Conf(id="context.meta",     name="System",   permission= "iface_view", interfaces=[IObject])
        ]
        
        self.objectheader = [
            Conf(id="object_info", interfaces=[IObject])
        ]

        self.shortcuts = [
            Conf(id="shortcut_add",     interfaces=[IContainer]), 
            Conf(id="shortcut_tools",   interfaces=[IObject]),
            Conf(id="shortcut_delete",  interfaces=[INonContainer])
        ]
        
        self.settings = [
        ]

        self.pageinfos = {}
        
        self.addflds =  Conf(default=Conf(fields=["title", AllTypeDataFlds], interfaces=[IObject]))
        self.editflds = Conf(default=Conf(fields=["title", AllTypeDataFlds], interfaces=[IObject]))
        self.metaflds = Conf(default=Conf(fields=["title", AllTypeMetaFlds], interfaces=[IObject]))
        self.dataflds = Conf(default=Conf(fields=["title", AllTypeDataFlds], interfaces=[IObject,IContainer]))
        
        # search and list defaults. can be replaced on type base.
        self.searchconf = Conf(default = 
            Conf(
                type = None,
                container = True,
                queryRestraints = None,
                callback = None, # callback for parameter lookup: callback(view, parameter, operators, searchconf)
                listoptions = ["ccp", "deletec"],
                form = ["pool_type", "title"],
                fields = ["pool_state","pool_type", "title", "pool_create","pool_createdby", "pool_change", "pool_changedby", "id"],
                method = None, # GET or POST
                max = 30,
                ascending = True,
                interfaces = [IContainer],
                renderer = None,
                options = None
            )
        )
            
        self.defaultviews= [
            Conf(id="viewlist", permission="iface_view", interfaces=[IContainer]),
            Conf(id="view", permission="iface_view", interfaces=[IObject]),
            Conf(id="base", permission="iface_view", interfaces=[IRoot])
        ]

        baseConf.__init__(self, copyFrom, **values)


    def __str__(self):
        return "%(id)s (%(name)s)" % self

    def test(self):
        report = []
        # check id
        if len(self.id) > 25:
            report.append((ConfigurationError, " ServiceConf.id too long. 25 chars allowed.", self))
        return report        
        

        
    

    



import time
import unittest
from pyramid import testing

from nive.portal import Portal
from nive.definitions import ConfigurationError
from nive.definitions import ModuleConf
from nive.helper import Event
from nive.definitions import OperationalError
from nive.tests.test_application import testapp, mApp2, mApp


class DummyClass1(object):
    def __init__(self, conf):
        pass
class DummyClass2(object):
    def hello(self):
        return "hello"
        

class portalTest(unittest.TestCase):
    
    def setUp(self):
        self.request = testing.DummyRequest()
        self.config = testing.setUp(request=self.request)
        self.portal = Portal()
        self.app = testapp(mApp2)

    def tearDown(self):
        self.app.Close()
        testing.tearDown()


    def test_register(self):
        self.portal.Register(mApp)
        self.portal.Register(self.app, "another")
        self.portal.Register("nive.tests.test_application.mApp2")
        self.assertRaises(ImportError, self.portal.Register, "nive.tests.test_application.mApp56875854")
        self.assertRaises(ConfigurationError, self.portal.Register, time)

    def test_module(self):
        testconf = ModuleConf(
            id = "module",
            name = "Module",
            context = "nive.tests.test_portal.DummyClass1",
            extensions = (DummyClass2,),
            events = None,
            description = ""
        )
        self.portal.Register(testconf, "test")
        m=self.portal["test"]
        self.assertTrue(self.portal["test"].hello()=="hello")

    def test_portal(self):
        self.portal.Register(mApp2)
        self.portal.Register(self.app, "nive")
        self.portal.RegisterGroups(self.app)
        self.assertTrue(self.portal.__getitem__("app2"))
        self.assertTrue(self.portal.__getitem__("nive"))
        try:
            self.portal.Startup(None)
        except OperationalError:
            pass
        self.assertTrue(len(self.portal.GetApps())==2)
        self.assertTrue(self.portal.GetGroups(sort="id", visibleOnly=False))
        self.assertTrue(self.portal.GetGroups(sort="name", visibleOnly=True))
        self.assertTrue(self.portal.portal)

    def test_portal2(self):
        self.portal.StartConnection(Event(self.request))
        self.portal.FinishConnection(Event(self.request))








"""
FormWorker for reform
---------------------
This is the connection between nive.Form and reform and maps FieldConf()
to schema nodes. 
"""

from pkg_resources import resource_filename
from nive.components.reform.template import ZPTRendererFactory

from nive.i18n import translate, _
from nive.definitions import ModuleConf, ViewModuleConf
from nive.helper import LoadListItems, ResolveName

from nive.components.reform.schema import *
from nive.components.reform.widget import *
from nive.components.reform.form import Button
import collections



# view module definition ------------------------------------------------------------------
#@nive_module
configuration = ModuleConf(
    id = "reformed",
    name = "Form widgets and resources",
    context = "nive.components.reform.reformed",
    views = (ViewModuleConf(static="nive.components.reform:static",id="reform"),),
)

template_dir = resource_filename('nive.components.reform', 'templates/')
zpt_renderer = ZPTRendererFactory([template_dir], translator=translate)



def SchemaFactory(form, fields, actions, force=False):
    """
    converts the fields to colander schema nodes including widget.
    If fielddef has node set and force is false node is used as widget. To overwrite set
    force = True.
    
    SchemaNode(...)
    """

    nodes = []
    for field in fields:
        # use field configuration node: node must be a valid 
        # SchemaNode 
        if field.get("node") and not force:
            nodes.append(field.node)
            continue
        
        # Default node setup
        # Basic values for all fields and widgets
        kw = {
            "name": field.id,
            "title": field.name,
            "description": field.description,
            "configuration": field
        }
        kwWidget = {
            "form": form,
            "configuration": field
        }
        if field.settings and isinstance(field.settings, dict):
            kwWidget.update(field.settings)

        # custom validator
        if field.get("validator"):
            kw["validator"] = field.validator
        # custom widget
        if field.get("widget"):
            kw["widget"] = field.widget

        # setup custom widgets
        if "widget" in kw and kw["widget"]:
            widget = kw["widget"]
            if isinstance(widget, str):
                widget = ResolveName(widget)
                kw["widget"] = widget(**kwWidget)
            elif isinstance(widget, collections.abc.Callable):
                kw["widget"] = widget(**kwWidget)
            else:
                widget.form = form
                widget.configuration = field
                kw["widget"] = widget

        # setup custom validator
        if "validator" in kw and kw["validator"]:
            validator = kw["validator"]
            if isinstance(validator, str):
                validator = ResolveName(validator)
            kw["validator"] = validator

        # setup missing and default value
        if not field.required:
            kw["missing"] = null # or "" ?
        if field.default is not None:
            kw["default"] = field.default

        if field.get("schema"):
            n = SchemaNode(field.get("schema"), **kw)
            
        elif field.hidden:
            n = hidden_node(field, kw, kwWidget, form)

        else:
            n = nodeMapping[field.datatype](*(field, kw, kwWidget, form))
            
        # add node to form
        if not n:
            continue
        nodes.append(n)
            
    # add buttons
    buttons = []
    for action in actions:
        if action.get("hidden"):
            continue
        buttons.append(Button(name="%s%s"%(action.get("id"), form.actionPostfix), 
                              title=action.get("name"), 
                              action=action, 
                              cls=action.get("cls", "btn submit")))

    # returns the prepared nodes and buttons
    return nodes, buttons


def hidden_node(field, kw, kwWidget, form):
    if not "widget" in kw:
        kw["widget"] = HiddenWidget(**kwWidget)
    return SchemaNode(String(), **kw)

def string_node(field, kw, kwWidget, form):
    if not "validator" in kw:
        kw["validator"] = Length(max=field.get("size",255))
    if not "widget" in kw:
        kw["widget"] = TextInputWidget(**kwWidget)
    return SchemaNode(String(), **kw)

def number_node(field, kw, kwWidget, form):
    if not "widget" in kw:
        kw["widget"] = TextInputWidget(**kwWidget)
    return SchemaNode(Integer(), **kw)

def float_node(field, kw, kwWidget, form):
    if not "widget" in kw:
        kw["widget"] = TextInputWidget(**kwWidget)
    return SchemaNode(Float(), **kw)

def bool_node(field, kw, kwWidget, form):
    if not "widget" in kw:
        v = LoadListItems(field, app=form.app, obj=form.context)
        if v:
            values=[(a["id"],a["name"]) for a in v]
        else:
            values = (("true", _("Yes")), ("false", _("No")))
        kw["widget"] = RadioChoiceWidget(values=values, **kwWidget)
    return SchemaNode(Boolean(), **kw)

def htext_node(field, kw, kwWidget, form):
    if not "validator" in kw:
        kw["validator"] = Length(max=field.get("size",1000000))
    if not "widget" in kw:
        kw["widget"] = RichTextWidget(**kwWidget)
    return SchemaNode(String(), **kw)

def text_node(field, kw, kwWidget, form):
    if not "validator" in kw:
        kw["validator"] = Length(max=field.get("size",1000000))
    if not "widget" in kw:
        kw["widget"] = TextAreaWidget(**kwWidget)
    return SchemaNode(String(), **kw)

def code_node(field, kw, kwWidget, form):
    if not "validator" in kw:
        kw["validator"] = Length(max=field.get("size",1000000))
    if not "widget" in kw:
        kw["widget"] = CodeTextWidget(**kwWidget)
    return SchemaNode(String(), **kw)

def json_node(field, kw, kwWidget, form):
    if not "validator" in kw:
        kw["validator"] = Length(max=field.get("size",1000000))
    if not "widget" in kw:
        kw["widget"] = CodeTextWidget(**kwWidget)
    return SchemaNode(JsonData(), **kw)

def file_node(field, kw, kwWidget, form):
    if not "widget" in kw:
        kw["widget"] = FileUploadWidget2(**kwWidget)
    return SchemaNode(FileData2(), **kw)
    #    kw["widget"] = FileUploadWidget(tmpstore, **kwWidget)
    #    n = SchemaNode(FileData(), **kw)

def date_node(field, kw, kwWidget, form):
    if not "widget" in kw:
        kw["widget"] = DateInputWidget(**kwWidget)
    return SchemaNode(Date(), **kw)

def datetime_node(field, kw, kwWidget, form):
    if not "widget" in kw:
        kw["widget"] = DateTimeInputWidget(**kwWidget)
    return SchemaNode(DateTime(form.app.pytimezone), **kw)

def time_node(field, kw, kwWidget, form):
    if not "widget" in kw:
        kw["widget"] = TextInputWidget(**kwWidget)
    return SchemaNode(Time(), **kw)

def list_node(field, kw, kwWidget, form):
    if hasattr(form, "view") and form.view is not None:
        user = form.view.User(sessionuser=False)
    else:
        user = None
    v = LoadListItems(field, app=form.app, obj=form.context, user=user)
    if field.settings and field.settings.get("addempty"):
        # copy the list and add empty entry
        v = list(v)
        v.insert(0,{"id":"","name":""})
    if not "widget" in kw:
        values = [(a["id"],a["name"]) for a in v]
        kw["widget"] = SelectWidget(values=values, **kwWidget)
        if field.settings.get("controlset"):
            kw["widget"].template = 'select_controlset'
            kw["widget"].css_class = ''
    return SchemaNode(CodeList(allowed=[str(e["id"]) for e in v]), **kw)

def radio_node(field, kw, kwWidget, form):
    v = LoadListItems(field, app=form.app, obj=form.context)
    if not "widget" in kw:
        values=[(a["id"],a["name"]) for a in v]
        kw["widget"] = RadioChoiceWidget(values=values, **kwWidget)
    return SchemaNode(CodeList(allowed=[str(e["id"]) for e in v]), **kw)

def multilist_node(field, kw, kwWidget, form):
    v = LoadListItems(field, app=form.app, obj=form.context)
    if not "widget" in kw:
        values=[(a["id"],a["name"]) for a in v]
        if field.settings.get("controlset"):
            kw["widget"] = SelectWidget(values=values, size=1, **kwWidget)
            kw["widget"].template = 'select_controlset'
            kw["widget"].css_class = ''
        else:
            kw["widget"] = SelectWidget(values=values, **kwWidget)
    return SchemaNode(List(allow_empty=True, allowed=[e["id"] for e in v]), **kw)

def checkbox_node(field, kw, kwWidget, form):
    v = LoadListItems(field, app=form.app, obj=form.context)
    if not "widget" in kw:
        values=[(a["id"],a["name"]) for a in v]
        kw["widget"] = CheckboxChoiceWidget(values=values, **kwWidget)
    return SchemaNode(List(allow_empty=True, allowed=[e["id"] for e in v]), **kw)

def lines_node(field, kw, kwWidget, form):
    if not "validator" in kw:
        kw["validator"] = Length(max=field.get("size",1000000))
    if not "widget" in kw:
        kw["widget"] = TextAreaWidget(**kwWidget)
    return SchemaNode(Lines(), **kw)

def email_node(field, kw, kwWidget, form):
    if not "validator" in kw:
        kw["validator"] = Email()
    if not "widget" in kw:
        kw["widget"] = TextInputWidget(**kwWidget)
    return SchemaNode(String(), **kw)

def url_node(field, kw, kwWidget, form):
    if not "validator" in kw:
        kw["validator"] = Length(max=field.get("size",255))
    if not "widget" in kw:
        kw["widget"] = TextInputWidget(**kwWidget, input_type="url")
    return SchemaNode(String(), **kw)

def urllist_node(field, kw, kwWidget, form):
    if not "validator" in kw:
        kw["validator"] = Length(max=field.get("size",1000000))
    if not "widget" in kw:
        kw["widget"] = TextAreaWidget(**kwWidget)
    return SchemaNode(Lines(), **kw)

def password_node(field, kw, kwWidget, form):
    if not "validator" in kw:
        kw["validator"] = Length(min=5, max=20)
    if not "widget" in kw:
        if field.settings.get("single",False):
            kw["widget"] = PasswordWidget(**kwWidget)
        else:
            if not "update" in kwWidget:
                kwWidget["update"] = field.settings.get("update",False)
            kw["widget"] = CheckedPasswordWidget(**kwWidget)
    return SchemaNode(String(), **kw)

def unit_node(field, kw, kwWidget, form):
    """
    options
    - add_title_id
    - obj_type
    - name_field
    - app
    - addempty
    - readonly_widget

    field.settings:
    :param obj_type: string or list. object type id.
    :param name_field: string. Codelist name lookup field. default = title.
    :param app: string. app id registered in portal.
    :param readonly_widget: bool. Show / hide modal button.
    """
    if not "validator" in kw and field.settings.get("app"):
        kw["validator"] = ExistingObject(obj_type=field.settings.get("obj_type"))
    if not "widget" in kw:
        if field.get("listItems") is None:
            ot = field.settings.get("obj_type")
            tf = field.settings.get("name_field", "title")
            operators = dict()
            parameter = dict()
            if ot:
                parameter["pool_type"] = ot
            if isinstance(ot, (list, tuple)):
                operators["pool_type"] = "IN"
            pool = form.app
            if field.settings.get("app"):
                pool = form.app.portal[field.settings.get("app")]
            values = pool.root.search.GetEntriesAsCodeList2(tf, parameter=parameter, operators=operators, sort=tf)
            if field.settings.get("add_title_id"):
                values = [(str(a["id"]),"%s (%d)"%(a["name"],a["id"])) for a in values]
            else:
                values = [(str(a["id"]),a["name"]) for a in values]
        else:
            if hasattr(form, "view"):
                user = form.view.User(sessionuser=False)
            else:
                user = None
            v = LoadListItems(field, app=form.app, obj=form.context, user=user)
            values = [(str(a["id"]), a["name"]) for a in v]

        if field.settings and field.settings.get("addempty"):
            # copy the list and add empty entry
            values.insert(0, ("", ""))
        kw["widget"] = UnitWidget(values=values, **kwWidget)
        if field.settings.get("readonly_widget"):
            kw["widget"].template = "unitselect_ro"
    return SchemaNode(Integer(), **kw)

def unitlist_node(field, kw, kwWidget, form):
    """
    options
    - add_title_id
    - obj_type
    - name_field
    - app

    field.settings:
    :param obj_type: string or list. object type id.
    :param name_field: string. Codelist name lookup field. default = title.
    :param app: string. app id registered in portal.
    """
    if not "widget" in kw:
        if field.get("listItems") is None:
            ot = field.settings.get("obj_type")
            tf = field.settings.get("name_field", "title")
            operators = dict()
            parameter = dict()
            if ot:
                parameter["pool_type"] = ot
            if isinstance(ot, (list, tuple)):
                operators["pool_type"] = "IN"
            pool = form.app
            if field.settings.get("app"):
                pool = form.app.portal[field.settings.get("app")]
            values = pool.root.search.GetEntriesAsCodeList2(tf, parameter=parameter, operators=operators, sort=tf)
            if field.settings.get("add_title_id"):
                values = [(str(a["id"]),"%s (%d)"%(a["name"],a["id"])) for a in values]
            else:
                values = [(str(a["id"]), a["name"]) for a in values]
        else:
            if hasattr(form, "view"):
                user = form.view.User(sessionuser=False)
            else:
                user = None
            v = LoadListItems(field, app=form.app, obj=form.context, user=user)
            values = [(str(a["id"]), a["name"]) for a in v]

        kw["widget"] = ChooseWidget(values=values, **kwWidget)
    return SchemaNode(List(allow_empty=True), **kw)

def timestamp_node(field, kw, kwWidget, form):
    # readonly
    return None

def nlist_node(field, kw, kwWidget, form):
    # unused
    return None

def binary_node(field, kw, kwWidget, form):
    # unused
    return None

nodeMapping = {
    "string": string_node,
    "number": number_node,
    "float": float_node,
    "bool": bool_node,
    "htext": htext_node,
    "text": text_node,
    "code": code_node,
    "json": json_node,
    "file": file_node,
    "date": date_node,
    "datetime": datetime_node,
    "time": time_node,
    "list": list_node,
    "radio": radio_node,
    "multilist": multilist_node,
    "checkbox": checkbox_node,
    "lines": lines_node,
    "email": email_node,
    "url": url_node,
    "urllist": urllist_node,
    "password": password_node,
    "unit": unit_node,
    "unitlist": unitlist_node,
    "timestamp": timestamp_node,
    "nlist": nlist_node,
    "binary": binary_node,
}




# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
# Released under GPL3. See license.txt
#

__doc__ = """
This file provides *object* functionality for objects_. The different components
are separated into classes. However these classes are not meant to be used as standalone. 

object.files values ::
        
    tag = file fldname
    filename = filename
    file = readable file object
    fileid = unique file id as number
    uid = fileid as string
    size = file size in bytes
    path = absolute local path
    mime = file mime type
    extension = file extension
    tempfile = True/False. If True file is not committed, yet.

"""

import weakref

from nive.definitions import implementer
from nive.definitions import StagContainer, ReadonlySystemFlds
from nive.definitions import IFieldConf, ICache, IContainer, INonContainer, IObject
from nive.definitions import ConfigurationError
from nive.workflow import WorkflowNotAllowed, ObjectWorkflow
from nive.security import SetupRuntimeAcls

from nive.events import Events



class ObjectRead(Events):
    """
    Object implementation with read access.
    """

    def __init__(self, id, dbEntry, parent, configuration, **kw):
        self.__name__ = str(id)
        self.__parent__ = parent
        self.id = id
        self.idhash = id
        self.path = str(id)  # ? unused
        self.dbEntry = dbEntry
        self.configuration = configuration
        self.selectTag = configuration.get("selectTag", StagContainer)

        # load security
        acl = SetupRuntimeAcls(configuration.acl, self)
        if acl:
            # omit if empty. acls will be loaded from parent object
            self.__acl__ = acl

        self.Signal("init")

    # Shortcuts -----------------------------------------------------------

    @property
    def meta(self):
        """ meta values """
        return self.dbEntry.meta

    @property
    def data(self):
        """ data values """
        return self.dbEntry.data

    @property
    def files(self):
        """ files """
        return self.dbEntry.files

    @property
    def root(self):
        """ returns the current root object in parent chain """
        return self.parent.root

    @property
    def app(self):
        """ returns the cms application itself """
        return self.root.app

    @property
    def parent(self):
        """ returns the parent object """
        return self.__parent__

    @property
    def db(self):
        """ returns the database object """
        return self.app.db

    @property
    def relpath(self):
        """ returns relative excluding root"""
        path = self.GetParentPaths()
        path.reverse()
        path.append(self.GetPath())
        return "/".join(path[1:])

    # Values ------------------------------------------------------

    def GetFld(self, fldname):
        """
        Get the meta/data value and convert to type.

        returns value or None
        """
        # data
        for f in self.configuration["data"]:
            if f["id"] == fldname:
                if f["datatype"] == "file":
                    return self.GetFile(fldname)
                return self.data.get(fldname)

        # meta
        f = self.app.configurationQuery.GetMetaFld(fldname)
        if f:
            return self.meta.get(fldname)
        return None

    def GetFile(self, fldname, loadFileData=False):
        """
        Get the file as `File` object with meta information. The included file
        pointer is opened on read.
        This functions directly accesses the database and ignores files cached in
        object.files.

        returns File object or None
        """
        return self.dbEntry.GetFile(fldname, loadFileData=loadFileData)

    def GetFileByName(self, filename):
        """
        Get a file by filename. See GetFile() for details.
        This functions directly accesses the database and ignores files cached in
        object.files.

        returns File object or None
        """
        file = self.dbEntry.Files(parameter={"filename": filename})
        if not len(file):
            return None
        return file[0]

    def GetFileByUID(self, uid):
        """
        Get a file by uid. Only for files contained in this object. See GetFile() for details.
        This functions directly accesses the database and ignores files cached in
        object.files.

        returns File object or none
        """
        file = self.dbEntry.Files(parameter={"fileid": uid})
        if not len(file):
            return None
        return file[0]

    def GetID(self):
        """
        Get the object id as number.

        returns number
        """
        return self.id

    def GetTypeID(self):
        """ returns the object type as string """
        return self.configuration["id"]

    def GetTypeName(self):
        """ returns the the object type name as string """
        return self.configuration["name"]

    def GetFieldConf(self, fldId):
        """
        Get the FieldConf for the field with id = fldId. Looks up data, file and meta
        fields.

        If `fldId` is a Field Configuration (``nive.definitions.FieldConf``) the functions
        checks if the field id is defined for the object.

        returns FieldConf or None
        """
        if IFieldConf.providedBy(fldId):
            f = [d for d in self.configuration.data if d["id"] == fldId.id]
            if f:
                return fldId
        else:
            f = [d for d in self.configuration.data if d["id"] == fldId]
            if f:
                return f[0]
        return self.app.configurationQuery.GetMetaFld(fldId)

    def GetTitle(self):
        """ returns the objects meta.title as string """
        return self.meta.get("title", "")

    def GetPath(self):
        """ returns the url path name of the object as string """
        return self.__name__

    # parents ----------------------------------------------------

    def IsRoot(self):
        """ returns bool """
        return False

    def GetParents(self):
        """ returns all parent objects as list """
        p = []
        o = self
        while o:
            o = o.parent
            if o:
                p.append(o)
        return p

    def GetParentIDs(self):
        """ returns all parent ids as list """
        p = []
        o = self
        while o:
            o = o.parent
            if o:
                p.append(o.id)
        return p

    def GetParentTitles(self):
        """ returns all parent titles as list """
        p = []
        o = self
        while o:
            o = o.parent
            if o:
                p.append(o.GetTitle())
        return p

    def GetParentPaths(self):
        """ returns all parent paths as list """
        p = []
        o = self
        while o:
            o = o.parent
            if o:
                p.append(o.GetPath())
        return p

    def IsContainer(self):
        """ returns if this object is a container """
        return IContainer.providedBy(self)

    # tools ----------------------------------------------------

    def GetTool(self, name):
        """
        Load a tool for execution in the objects context. Only works for tools applied
        to this objects type. ::

            returns the tool or None

        Event
        - loadToool(tool=toolObj)
        """
        t = self.app.GetTool(name, self)
        self.Signal("loadTool", tool=t)
        return t

    def Close(self):
        """
        Close the object and all contained objects. Currently only used in combination with caches.

        Event
        - close()
        """
        self.Signal("close")
        if ICache.providedBy(self):
            # opt
            for o in self.GetAllFromCache():
                o.Close()
            p = self.parent
            if ICache.providedBy(p):
                p.RemoveCache(self.id)
        self.dbEntry.Close()




class ObjectWrite:
    """
    Provides *edit* functionality for objects.

    Requires (Object, ObjectWorkflow)
    """

    @property
    def workflow(self):
        return  ObjectWorkflow(self)

    def SplitData(self, sourceData):
        """
        Split sourceData dictionary in data, meta and file based on this objects
        configuration. Unused fields in source data are ignored.
        
        returns data, meta, files (each as dictionary)
        """
        data = {}
        meta = {}
        files = {}
        for f in self.configuration["data"]:
            id = f["id"]
            if id in sourceData:
                if f["datatype"]=="file":
                    files[id] = sourceData[id]
                else:
                    data[id] = sourceData[id]
        for f in self.app.configurationQuery.GetAllMetaFlds(False):
            id = f["id"]
            if id in sourceData:
                meta[id] = sourceData[id]
        return data, meta, files


    def Commit(self, user):
        """
        Commit changes made to data, meta and file attributes and
        calls workflow "edit" action.
        
        Event: 
        - commit()
        
        Workflow action: edit
        """
        wf = self.workflow
        # check workflow
        if not wf.WfAllow("edit", user=user):
            raise WorkflowNotAllowed("Workflow: Not allowed (edit)")
        self.CommitInternal(user=user)
        # call wf
        try:
            wf.WfAction("edit", user=user)
        except Exception as e:
            self.Undo()
            raise 
        return True


    def Undo(self):
        """
        Undo changes made to data, meta and file attributes
        
        Event: 
        - undo()
        """
        self.Signal("undo")
        self.dbEntry.Undo()


    def Update(self, data, user):
        """
        Updates data, commits and calls workflow "edit" action.
        *data* can contain data, meta and files. ::
        
            data = dictionary containing data, meta and files
            user = the current user
            returns bool 
        
        Events:
         
        - update(data)
        - commit()
        
        Workflow action: edit
        """
        app = self.app
        wf = self.workflow
        # check workflow
        if not wf.WfAllow("edit", user=user):
            raise WorkflowNotAllowed("Workflow: Not allowed (edit)")
        self.Signal("update", data=data)
        self.UpdateInternal(data)
        # call wf
        try:
            wf.WfAction("edit", user=user)
        except Exception as e:
            self.Undo()
            raise 

        if app.configuration.autocommit:
            self.CommitInternal(user=user)
        return True


    def StoreFile(self, fldname, file, user):
        """
        Store a file under fldname. Existing files will be replaced. ::
        
            fldname = field name to store the file
            file = the file to store as FileObject
            user = the current user
            returns bool
        
        Events: 
        
        - storeFile(filename, fldname)
        
        Workflow action: edit
        """
        app = self.app
        wf = self.workflow
        # check workflow
        if not wf.WfAllow("edit", user=user):
            raise WorkflowNotAllowed("Workflow: Not allowed (edit)")
        self.Signal("storeFile", file=file, fldname=fldname)
        self.dbEntry.CommitFile(fldname, file)
        # call wf
        try:
            wf.WfAction("edit", user=user)
        except Exception as e:
            self.Undo()
            raise 

        if app.configuration.autocommit:
            self.db.Commit()
        return True


    def DeleteFile(self, fldname, user):
        """
        Delete a file from this object. ::

            fldname = field name to store the file
            user = the current user
            returns bool

        Event: 
        - deleteFile(fldname)
        
        Workflow action: edit
        """
        if not fldname:
            return False
        app = self.app
        wf = self.workflow
        # check workflow
        if not wf.WfAllow("edit", user=user):
            raise WorkflowNotAllowed("Workflow: Not allowed (edit)")
        self.Signal("deleteFile", fldname=fldname)
        result = self.dbEntry.DeleteFile(fldname)
        if not result:
            return False
        # call wf
        try:
            wf.WfAction("edit", user=user)
        except Exception as e:
            self.Undo()
            raise 

        if app.configuration.autocommit:
            self.CommitInternal(user=user)
        return True


    def RenameFile(self, filekey, filename, user):
        """
        Changes the filename field of the file `filekey`. ::

            filekey = file to set the new filename for
            filename = new filename
            user = the current user
            returns bool

        -
        """
        if self.dbEntry.RenameFile(filekey, filename):
            self.dbEntry.Touch()
            self.Commit(user)
            return True
        return False
    
    
    # Edit functions without workflow ------------------------------------------

    def UpdateInternal(self, data):
        """
        Update this objects data without calling workflow actions or events
        
        returns bool
        """
        data, meta, files = self.SplitData(data)
        # delete system fields
        system = ReadonlySystemFlds
        #opt
        if meta:
            for f in system:
                if f in meta:
                    del meta[f]
        # store data
        self.meta.update(meta)
        self.data.update(data)
        self.files.update(files)
        return True


    def CommitInternal(self, user):
        """
        Commit changes made to data, meta and files attributes without calling wf 
        
        Event: 
        - commit(user)
        """
        self.Signal("commit", user=user)
        self.dbEntry.Commit(user=user)


    def CreateSelf(self, data, user, **kw):
        """
        Called after the new object is created.
        
        Event: 
        - wfInit(processID)

        Workflow action: create
        """
        # initialize workflow
        wf = self.workflow
        wf.WfInit(user)

        # split data for pool
        self.UpdateInternal(data)
        self.meta["pool_type"] = self.configuration.id
        self.meta["pool_unitref"] = self.parent.id
        self.meta["pool_stag"] = self.selectTag

        # call wf
        try:
            wf.WfAction("create", user=user)
        except Exception as e:
            self.Undo()
            raise 

        return



@implementer(INonContainer, IObject)
class Object(ObjectRead, ObjectWrite):
    """
    Object implementation with read and write access.
    """

    pass


# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
# Released under GPL3. See license.txt
#
__doc__ = """
Application.modules stores configuration objects registered by calling `RegisterComponents()`. 
"""

import uuid
from zope.interface import providedBy

from nive.definitions import IViewModuleConf, IViewConf, IRootConf, IObjectConf, IToolConf
from nive.definitions import IAppConf, IDatabaseConf, IModuleConf, IWidgetConf, IFieldConf
from nive.definitions import ReadonlySystemFlds
from nive.definitions import ConfigurationError

from nive.utils.utils import SortConfigurationList
from nive.helper import ResolveName, ResolveConfiguration, FormatConfTestFailure
from nive.helper import DecorateViewClassWithViewModuleConf
from nive.tool import _IGlobal, _GlobalObject
from nive.workflow import IWfProcessConf


class AppConfigurationQuery(object):
    """
    Read access functions for root, type, type field, meta field and category configurations.

    Requires:
    - nive.nive
    """
    def __init__(self, app):
        self.app = app

    def QueryConf(self, queryFor, context=None):
        """
        Returns a list of configurations or empty list
        """
        if isinstance(queryFor, str):
            queryFor = ResolveName(queryFor, raiseExcp=False)
            if not queryFor:
                return ()
        if context:
            return self.app.registry.getAdapters((context,), queryFor)
        return self.app.registry.getAllUtilitiesRegisteredFor(queryFor)

    def QueryConfByName(self, queryFor, name, context=None):
        """
        Returns configuration or None
        """
        if isinstance(queryFor, str):
            queryFor = ResolveName(queryFor, raiseExcp=False)
            if not queryFor:
                return None
        if context:
            v = self.app.registry.queryAdapter(context, queryFor, name=name)
            if v:
                return v[0]
            return None
        return self.app.registry.queryUtility(queryFor, name=name)

    # Roots -------------------------------------------------------------

    def GetRootConf(self, name=""):
        """
        Get the root object configuration. If name is empty, the default name is used.

        returns dict or None
        """
        if name == "":
            name = self.app.rootname
        return self.app.registry.queryUtility(IRootConf, name=name)

    def GetAllRootConfs(self):
        """
        Get all root object configurations as list.

        returns list
        """
        return self.app.registry.getAllUtilitiesRegisteredFor(IRootConf)


    # Types -------------------------------------------------------------

    def GetObjectConf(self, typeID, skipRoot=False):
        """
        Get the type configuration for typeID. If type is not found root definitions are
        searched as well.

        returns configuration or none
        """
        c = self.app.registry.queryUtility(IObjectConf, name=typeID)
        if c or skipRoot:
            return c
        return self.GetRootConf(typeID)

    def GetAllObjectConfs(self, visibleOnly=False):
        """
        Get all type configurations.

        returns list
        """
        c = self.app.registry.getAllUtilitiesRegisteredFor(IObjectConf)
        if not visibleOnly:
            return c
        return [t for t in c if t.get("hidden") != True]

    def GetTypeName(self, typeID):
        """
        Get the object type name for the id.

        returns string
        """
        t = self.GetObjectConf(typeID)
        if t:
            return t.name
        return ""

    # General Fields -------------------------------------------------------------

    def GetFld(self, fldID, typeID=None):
        """
        Get type or meta field definition. Type fields have the priority if typeID is not None.

        returns configuration or None
        """
        if typeID is not None:
            f = self.GetObjectFld(fldID, typeID)
            if f is not None:
                return f
        return self.GetMetaFld(fldID)

    def GetFldName(self, fldID, typeID=None):
        """
        Get type or meta field name. Type fields have the priority if typeID is not None.

        returns string
        """
        if typeID:
            f = self.GetObjectFld(fldID, typeID)
            if f:
                return f["name"]
        return self.GetMetaFldName(fldID)

    # Type Data Fields -------------------------------------------------------------

    def GetObjectFld(self, fldID, typeID):
        """
        Returns object field configuration.

        returns configuration or None
        """
        fields = self.GetAllObjectFlds(typeID)
        if not fields:
            return None
        if IFieldConf.providedBy(fldID):
            f = [d for d in fields if d["id"] == fldID.id]
            if f:
                return fldID
        else:
            f = [d for d in fields if d["id"] == fldID]
            if f:
                return f[0]
        return None

    def GetAllObjectFlds(self, typeID):
        """
        Get all object field configurations.

        returns list or None
        """
        t = self.GetObjectConf(typeID)
        if not t:
            return None
        return t.data

    # Meta Fields -------------------------------------------------------------

    def GetMetaFld(self, fldID):
        """
        Get meta field configuration

        returns configuration or None
        """
        if IFieldConf.providedBy(fldID):
            f = [d for d in self.app.configuration.meta if d["id"] == fldID.id]
            if f:
                return fldID
        else:
            f = [d for d in self.app.configuration.meta if d["id"] == fldID]
            if f:
                return f[0]
        return None

    def GetAllMetaFlds(self, ignoreSystem=True):
        """
        Get all meta field configurations.

        returns list
        """
        if not ignoreSystem:
            return self.app.configuration.meta
        return [m for m in self.app.configuration.meta if m["id"] not in ReadonlySystemFlds]

    def GetMetaFldName(self, fldID):
        """
        Get meta field name for id.

        returns string
        """
        m = [d for d in self.app.configuration.meta if d["id"] == fldID]
        if not m:
            return ""
        return m[0]["name"]

    # Tool -------------------------------------------------------------

    def GetToolConf(self, toolID, contextObject=None):
        """
        Get the tool configuration.

        returns configuration or None
        """
        if not contextObject:
            contextObject = _GlobalObject()
        v = self.app.registry.queryAdapter(contextObject, IToolConf, name=toolID)
        if v:
            return v[0]
        return None

    def GetAllToolConfs(self, contextObject=None):
        """
        Get all tool configurations.

        returns list
        """
        tools = []
        for a in self.app.registry.registeredAdapters():
            if a.provided == IToolConf:
                if contextObject:
                    if not a.required[0] in providedBy(contextObject):
                        continue
                tools.append(a.factory)
        return tools

    # Categories -------------------------------------------------------------------------------------------------------

    def GetCategory(self, categoryID=""):
        """
        Get category configuration.

        returns configuration or None
        """
        c = [d for d in self.app.configuration.categories if d["id"] == categoryID]
        if not c:
            return None
        return c[0]

    def GetAllCategories(self, sort="name", visibleOnly=False):
        """
        Get all category configurations.

        returns list
        """
        if not visibleOnly:
            return SortConfigurationList(self.app.configuration.categories, sort)
        c = [a for a in self.app.configuration.categories if not a.get("hidden")]
        return SortConfigurationList(c, sort)

    def GetCategoryName(self, categoryID):
        """
        Get category name for id.

        returns string
        """
        c = [d for d in self.app.configuration.categories if d["id"] == categoryID]
        if not c:
            return ""
        return c[0]["name"]

    # Workflow -------------------------------------------------------------

    def GetWorkflowConf(self, processID, contextObject=None):
        """
        Get the workflow process configuration.

        returns configuration or None
        """
        v = self.app.registry.queryAdapter(contextObject or _GlobalObject(), IWfProcessConf, name=processID)
        if v:
            return v[0]
        return None

    def GetAllWorkflowConfs(self, contextObject=None):
        """
        Get all workflow configurations.

        returns list
        """
        w = []
        adpts = self.app.registry.registeredAdapters()
        for a in adpts:
            if a.provided == IWfProcessConf:
                if contextObject:
                    if not a.required[0] in providedBy(contextObject):
                        continue
                w.append(a.factory)
        return w


class Registration(object):

    def __init__(self, app):
        self.app = app
        self.log = app.log
        

    def RegisterComponents(self, module, **kw):
        """
        Include a module or configuration to store in the registry.

        Handles configuration objects with the following interfaces:

        - IAppConf
        - IRootConf
        - IObjectConf
        - IViewModuleConf
        - IViewConf 
        - IToolConf
        - IModuleConf
        - IWidgetConf
        - IWfProcessConf

        Other modules are registered as utility with **kws as parameters.

        raises TypeError, ConfigurationError, ImportError
        """
        app = self.app
        log = self.log
        iface, conf = ResolveConfiguration(module)
        if not conf:
            log.debug('Register python module: %s', str(module))
            return app.registry.registerUtility(module, **kw)

        # test conf
        if app.debug:
            r = conf.test()
            if r:
                v = FormatConfTestFailure(r)
                log.warning('Configuration test failed:\r\n%s', v)
                # return False

        log.debug('Register module: %s %s', str(conf), str(iface))
        # register module views
        if iface not in (IViewModuleConf, IViewConf):
            self._RegisterConfViews(conf)

        # reset cached class value. makes testing easier
        try:
            del conf._c_class
        except:
            pass

        # register module itself
        if hasattr(conf, "unlock"):
            conf.unlock()

        # events
        if conf.get("events"):
            for e in conf.events:
                log.debug('Register Event: %s for %s', str(e.event), str(e.callback))
                app.ListenEvent(e.event, e.callback)

        if iface == IRootConf:
            app.registry.registerUtility(conf, provided=IRootConf, name=conf.id)
            if conf.default or not app._defaultRoot:
                app._defaultRoot = conf.id
            return True
        elif iface == IObjectConf:
            app.registry.registerUtility(conf, provided=IObjectConf, name=conf.id)
            return True

        elif iface == IViewConf:
            app.registry.registerUtility(conf, provided=IViewConf, name=conf.id or str(uuid.uuid4()))
            return True
        elif iface == IViewModuleConf:
            app.registry.registerUtility(conf, provided=IViewModuleConf, name=conf.id)
            if conf.widgets:
                for w in conf.widgets:
                    self.RegisterComponents(w)
            return True

        elif iface == IToolConf:
            if conf.apply:
                for i in conf.apply:
                    if i is None:
                        app.registry.registerAdapter(conf, (_IGlobal,), IToolConf, name=conf.id)
                    else:
                        app.registry.registerAdapter(conf, (i,), IToolConf, name=conf.id)
            else:
                app.registry.registerAdapter(conf, (_IGlobal,), IToolConf, name=conf.id)
            if conf.modules:
                for m in conf.modules:
                    self.RegisterComponents(m, **kw)
            return True

        elif iface == IAppConf:
            app.registry.registerUtility(conf, provided=IAppConf, name="IApp")
            if conf.modules:
                for m in conf.modules:
                    self.RegisterComponents(m)
            return True

        elif iface == IDatabaseConf:
            app.registry.registerUtility(conf, provided=IDatabaseConf, name="IDatabase")
            app.dbConfiguration = conf
            return True

        elif iface == IModuleConf:
            # modules
            if conf.modules:
                for m in conf.modules:
                    self.RegisterComponents(m, **kw)
            app.registry.registerUtility(conf, provided=IModuleConf, name=conf.id)
            return True

        elif iface == IWidgetConf:
            for i in conf.apply:
                app.registry.registerAdapter(conf, (i,), conf.widgetType, name=conf.id)
            return True

        elif iface == IWfProcessConf:
            if conf.apply:
                for i in conf.apply:
                    app.registry.registerAdapter(conf, (i,), IWfProcessConf, name=conf.id)
            else:
                app.registry.registerAdapter(conf, (_IGlobal,), IWfProcessConf, name=conf.id)
            return True

        raise TypeError("Unknown configuration interface type (%s)" % (str(conf)))

    def RegisterViews(self, config):
        """
        Register configured views and static views with the pyramid web framework.
        """
        app = self.app
        ref = str(app.configuration.context)
        predicatename = ref+"_AppContainmentPredicate"
        config.add_view_predicate(predicatename, AppContainmentPredicate)

        views = app.registry.getAllUtilitiesRegisteredFor(IViewConf)
        # single views
        for view in views:
            opts = {predicatename: ref}
            if view.options:
                opts.update(view.options)

            config.add_view(view=view.view,
                            context=view.context,
                            attr=view.attr,
                            name=view.name,
                            renderer=view.renderer,
                            permission=view.permission,
                            containment=view.containment,
                            **opts)
        return config

    def RegisterViewModules(self, config):
        """
        Register configured views and static views with the pyramid web framework.
        """
        app = self.app
        ref = str(app.configuration.context)
        predicatename = ref+"_AppContainmentPredicate"
        config.add_view_predicate(predicatename, AppContainmentPredicate)

        mods = app.registry.getAllUtilitiesRegisteredFor(IViewModuleConf)
        for viewmod in mods:
            # decorate viewmod to add a pointer to the view module configuration. otherwise there is no way
            # to access the configuration from inside the view callable
            viewcls = viewmod.view
            if viewcls:
                viewcls = DecorateViewClassWithViewModuleConf(viewmod, viewcls)
            # object views
            for view in viewmod.views:
                opts = {predicatename: ref}
                if view.options:
                    opts.update(view.options)

                config.add_view(attr=view.attr,
                                name=view.name,
                                view=view.view or viewcls,
                                context=view.context or viewmod.context,
                                renderer=view.renderer or viewmod.renderer,
                                permission=view.permission or viewmod.permission,
                                containment=view.containment or viewmod.containment,
                                **opts)

            # static views
            maxage = 60 * 60 * 4
            if app.debug:
                maxage = None
            if isinstance(viewmod.static, (list,tuple)):
                for static in viewmod.static:
                    if isinstance(static, dict):
                        config.add_static_view(name=static["name"], path=static["path"], cache_max_age=static.get("maxage", maxage))
                    else:
                        config.add_static_view(name=static[0], path=static[1], cache_max_age=viewmod.get("maxage", maxage))

            elif isinstance(viewmod.static, dict):
                config.add_static_view(name=static["name"], path=static["path"], cache_max_age=static.get("maxage", maxage))

            elif viewmod.static:
                # bw update
                config.add_static_view(name=viewmod.staticName or viewmod.id, path=viewmod.static, cache_max_age=maxage)

            # acls
            if viewmod.get("acl"):
                app.__acl__ = tuple(list(viewmod.get("acl")) + list(app.__acl__))

        return config

    def RegisterTranslations(self, config):
        """
        Register translation directories contained in configurations.
        Translations can be specified as `configuraion.translations` for 
        IAppConf, IViewModuleConf, IObjectConf, IRootConf, IToolConf, IModuleConf, IWidgetConf
        """
        for confType in (IAppConf, IViewModuleConf, IObjectConf, IRootConf, IToolConf, IModuleConf, IWidgetConf):
            mods = self.app.registry.getAllUtilitiesRegisteredFor(confType)
            for modconf in mods:
                path = modconf.translations
                # translations
                if isinstance(path, str):
                    config.add_translation_dirs(path)
                elif isinstance(path, (list, tuple)):
                    config.add_translation_dirs(*path)

    def _RegisterConfViews(self, conf):
        """
        Register view configurations included as ``configuration.views`` in other
        configuration classes like ObjectConf, RootConf and so on.
        """
        views = None
        try:
            views = conf.views
        except:
            pass
        if not views:
            return
        for v in views:

            if isinstance(v, str):
                iface, conf = ResolveConfiguration(v)
                if not conf:
                    raise ConfigurationError(str(v))
                v = conf

            self.RegisterComponents(v)


class AppContainmentPredicate(object):
    """
    Check if context of view is this application. For multisite support.
    """
    def __init__(self, val, config):
        self.ref = val

    def __call__(self, context, request):
        return str(context.app.configuration.context) == self.ref

    def text(self):
        return self.ref+"_AppContainmentPredicate"
    phash = text

